@@ -1,8 +1,7 @@
-
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -396,7 +395,7 @@ function form_actions() {
 				print "<tr>
 					<td class='textArea'>
 						<p>" . __('The selected Aggregate Graphs does not appear to have any matching Aggregate Templates.') . "</p>
-						<p>" . __('In order to migrate the Aggregate Graphs below use an Aggregate Template, one must already exist.  Please press \'Return\' and then first create your Aggergate Template before retrying.') . "</p>
+						<p>" . __('In order to migrate the Aggregate Graphs below use an Aggregate Template, one must already exist.  Please press \'Return\' and then first create your Aggregate Template before retrying.') . "</p>
 						<div class='itemlist'><ul>$graph_list</ul></div>
 					</td>
 				</tr>";
@@ -487,7 +486,7 @@ function form_actions() {
 				print "<tr><td class='even'><span class='textError'>" . __('You currently have no reports defined.') . "</span></td></tr>";
 				$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Return') . "' onClick='cactiReturnTo()'>";
 			}
-		} elseif (get_request_var('drp_action') == '5') { // conver to a normal graph
+		} elseif (get_request_var('drp_action') == '5') { // convert to a normal graph
 			print "<tr>
 				<td colspan='2' class='textArea'>
 					<p>" . __('Click \'Continue\' to convert the following Aggregate Graph(s) into a normal Graph.') . "</p>
@@ -556,6 +555,9 @@ function form_actions() {
 function item() {
 	global $consolidation_functions, $graph_item_types, $struct_graph_item;
 
+	// Remove filter item
+	unset($struct_graph_item['data_template_id']);
+
 	/* ================= input validation ================= */
 	get_filter_request_var('id');
 	/* ==================================================== */
@@ -614,6 +616,9 @@ function item() {
 function graph_edit() {
 	global $config, $struct_graph, $struct_aggregate_graph, $image_types, $consolidation_functions, $graph_item_types, $struct_graph_item;
 
+	// Remove filter item
+	unset($struct_graph_item['data_template_id']);
+
 	/* ================= input validation ================= */
 	get_filter_request_var('id');
 	/* ==================================================== */
@@ -765,7 +770,7 @@ function graph_edit() {
 
 		if (!cacti_sizeof($graph)) {
 			html_start_box(__('Aggregate Preview Does Not Exist'), '100%', '', '3', 'center', '');
-			print "<tr><td id='imagewindow' class='center'>" . __('Aggreage Graph does not Exist') . '</tr></tr>';
+			print "<tr><td id='imagewindow' class='center'>" . __('Aggregate Graph does not Exist') . '</tr></tr>';
 			html_end_box(false);
 			raise_message('noaggregate', __('Aggregate Graph does not Exist'), MESSAGE_LEVEL_ERROR);
 			return false;
@@ -795,9 +800,9 @@ function graph_edit() {
 				</div>
 				<script type='text/javascript'>
 				$(function() {
-					var width = $(window).width() - $('.cactiConsoleNavigationArea').width();
+					var rrdwidth = $(window).width() - $('.cactiConsoleNavigationArea').width();
 					$('#agg_preview').show();
-					$('#rrdtoolinfo, #imagewindow').css('max-width', width);
+					$('#rrdtoolinfo, #imagewindow').css('max-width', rrdwidth);
 				});
 				</script>
 			</td></tr>
@@ -816,7 +821,7 @@ function graph_edit() {
 
 	form_start('aggregate_graphs.php', 'template_edit');
 
-	/* we will show the templated representation only when when there is a template and propagation is enabled */
+	/* we will show the templated representation only when there is a template and propagation is enabled */
 	if (!isempty_request_var('id') && $current_tab == 'details') {
 		if (cacti_sizeof($template)) {
 			print "<div id='templated'>";
@@ -1221,7 +1226,7 @@ function aggregate_items() {
 		$sql_where .= ($sql_where != '' ? ' AND ':'WHERE ') . ' agi.local_graph_id IN(' . get_request_var('local_graph_ids') . ')';
 	}
 
-	$total_rows = db_fetch_cell("SELECT COUNT(DISTINCT gl.id) AS total
+	$sql = "SELECT COUNT(DISTINCT gl.id) AS total
 		FROM graph_templates_graph AS gtg
 		INNER JOIN graph_local AS gl
 		ON gtg.local_graph_id=gl.id
@@ -1230,7 +1235,9 @@ function aggregate_items() {
 			FROM aggregate_graphs_items
 			WHERE aggregate_graph_id=$aggregate_id) AS agi
 		ON gtg.local_graph_id=agi.local_graph_id
-		$sql_where");
+		$sql_where";
+
+	$total_rows = get_total_row_data($_SESSION['sess_user_id'], $sql, array(), 'aggregate_graph');
 
 	$sql_order = get_order_string();
 	$sql_limit = ' LIMIT ' . ($rows*(get_request_var('page')-1)) . ',' . $rows;
@@ -1249,7 +1256,6 @@ function aggregate_items() {
 		$sql_order
 		$sql_limit");
 
-
 	?>
 	<script type='text/javascript'>
 	var totalItems=<?php print $total_items;?>;
@@ -1274,10 +1280,6 @@ function aggregate_items() {
 			$('#agg_preview').show();
 		}
 
-		$('#refresh').click(function() {
-			applyFilter();
-		});
-
 		$('#clear').click(function() {
 			clearFilter();
 		});
@@ -1286,7 +1288,7 @@ function aggregate_items() {
 			applyFilter();
 		});
 
-		$('#form_graphs').submit(function(event) {
+		$('#forms').submit(function(event) {
 			event.preventDefault();
 			applyFilter();
 		});
@@ -1299,7 +1301,7 @@ function aggregate_items() {
 	?>
 	<tr class='even'>
 		<td>
-			<form id='form_graphs' action='aggregate_graphs.php'>
+			<form id='forms' action='aggregate_graphs.php'>
 			<table class='filterTable'>
 				<tr>
 					<td>
@@ -1331,7 +1333,7 @@ function aggregate_items() {
 					</td>
 					<td>
 						<span>
-							<input type='button' class='ui-button ui-corner-all ui-widget' id='refresh' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
+							<input type='submit' class='ui-button ui-corner-all ui-widget' id='go' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
 							<input type='button' class='ui-button ui-corner-all ui-widget' id='clear' onClick='clearFilter()' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>'>
 						</span>
 					</td>
@@ -1555,19 +1557,11 @@ function aggregate_graph() {
 			$('#agg_preview').show();
 		}
 
-		$('#refresh').click(function() {
-			applyFilter();
-		});
-
 		$('#clear').click(function() {
 			clearFilter();
 		});
 
-		$('#filter').change(function() {
-			applyFilter();
-		});
-
-		$('#form_graphs').submit(function(event) {
+		$('#forms').submit(function(event) {
 			event.preventDefault();
 			applyFilter();
 		});
@@ -1581,7 +1575,7 @@ function aggregate_graph() {
 	?>
 	<tr class='even'>
 		<td>
-			<form id='form_graphs' action='aggregate_graphs.php'>
+			<form id='forms' action='aggregate_graphs.php'>
 			<table class='filterTable'>
 				<tr>
 					<td>
@@ -1629,7 +1623,7 @@ function aggregate_graph() {
 					</td>
 					<td>
 						<span>
-							<input type='button' class='ui-button ui-corner-all ui-widget' id='refresh' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
+							<input type='submit' class='ui-button ui-corner-all ui-widget' id='go' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
 							<input type='button' class='ui-button ui-corner-all ui-widget' id='clear' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>'>
 						</span>
 					</td>
@@ -1657,7 +1651,7 @@ function aggregate_graph() {
 		$sql_where .= ' AND ag.aggregate_template_id=' . get_request_var('template_id');
 	}
 
-	$total_rows = db_fetch_cell("SELECT COUNT(DISTINCT gl.id) AS total
+	$sql = "SELECT COUNT(DISTINCT gl.id) AS total
 		FROM graph_templates_graph AS gtg
 		INNER JOIN graph_local AS gl
 		ON gtg.local_graph_id=gl.id
@@ -1665,7 +1659,9 @@ function aggregate_graph() {
 		ON gtg.local_graph_id=ag.local_graph_id
 		LEFT JOIN aggregate_graph_templates AS agt
 		ON agt.id=ag.aggregate_template_id
-		$sql_where");
+		$sql_where";
+
+	$total_rows = get_total_row_data($_SESSION['sess_user_id'], $sql, array(), 'aggregate_graph');
 
 	$sql_order = get_order_string();
 	$sql_limit = ' LIMIT ' . ($rows*(get_request_var('page')-1)) . ',' . $rows;
