@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -28,7 +28,7 @@ chdir($dir);
 
 /* Start Initialization Section */
 require(__DIR__ . '/../include/cli_check.php');
-include_once($config['base_path'] . '/lib/spikekill.php');
+require_once($config['base_path'] . '/lib/spikekill.php');
 
 if ($config['poller_id'] > 1) {
 	print "FATAL: This utility is designed for the main Data Collector only" . PHP_EOL;
@@ -220,7 +220,7 @@ function display_help () {
 	print "    [-P|--percent=N] [-N|--number=N] [-D|--dryrun] [-d|--debug]\n";
 	print "    [--html]\n\n";
 
-	print "A utility to programatically remove spikes from Cacti graphs. If no optional input parameters\n";
+	print "A utility to programmatically remove spikes from Cacti graphs. If no optional input parameters\n";
 	print "are specified the defaults are taken from the Cacti database.\n\n";
 
 	print "Required:\n";
