@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -23,42 +23,16 @@
  +-------------------------------------------------------------------------+
 */
 
+require_once(__DIR__ . '/include/cli_check.php');
 if (function_exists('pcntl_async_signals')) {
 	pcntl_async_signals(true);
 } else {
 	declare(ticks = 100);
 }
 
-function sig_handler($signo) {
-	switch ($signo) {
-		case SIGTERM:
-		case SIGINT:
-			cacti_log('WARNING: Cacti Poller process terminated by user', true, 'POLLER');
-
-			// record the process as having completed
-			record_cmdphp_done();
-			db_close();
-
-			exit;
-			break;
-		default:
-			// ignore all other signals
-	}
-}
-
-require_once(__DIR__ . '/include/cli_check.php');
-require_once($config['base_path'] . '/lib/snmp.php');
-require_once($config['base_path'] . '/lib/poller.php');
-require_once($config['base_path'] . '/lib/rrd.php');
-require_once($config['base_path'] . '/lib/ping.php');
-
-global $poller_id, $sessions, $downhosts, $remote_db_cnn_id, $print_data_to_stdout;
-
 ini_set('max_execution_time', '0');
 ini_set('memory_limit', '-1');
 
-$no_http_headers = true;
-
 // process calling arguments
 $parms = $_SERVER['argv'];
 array_shift($parms);
@@ -74,9 +48,11 @@ $downhosts  = array();
 $host_count = 0;
 $tot_errors = 0;
 $poller_id  = $config['poller_id'];
-$maxwidth   = get_max_column_width();
+$pmessage   = false;
+$help       = false;
+$version    = false;
 
-if (cacti_sizeof($parms)) {
+if (sizeof($parms)) {
 	foreach($parms as $parameter) {
 		if (strpos($parameter, '=')) {
 			list($arg, $value) = explode('=', $parameter);
@@ -89,49 +65,77 @@ if (cacti_sizeof($parms)) {
 		case '--version':
 		case '-V':
 		case '-v':
-			display_version();
-			exit;
+			$version = true;
+
+			break;
 		case '--help':
 		case '-H':
 		case '-h':
-			display_help();
-			exit;
+			$help = true;
+
+			break;
 		case '--poller':
 		case '-p':
-			cacti_log('Forcing poller to ' . $value, true, 'POLLER', POLLER_VERBOSITY_HIGH);
+			$pmessage = true;
 			$poller_id = $value;
+
 			break;
 		case '--first':
 		case '-f':
 			$first   = $value;
 			$allhost = false;
+
 			break;
 		case '--last':
 		case '-l':
 			$last    = $value;
 			$allhost = false;
+
 			break;
 		case '--mibs':
 		case '-m':
-			$mibs    = true;
+			$mibs = true;
+
 			break;
 		case '--mode':
 		case '-N':
 			$mode = $value;
+
 			break;
 		case '--debug':
 		case '-d':
 			$debug = true;
+
 			break;
 		default:
-			print "ERROR: Invalid Argument: ($arg)\n\n";
+			print "ERROR: Invalid Argument: ($arg)" . PHP_EOL . PHP_EOL;
 			display_help();
 			exit(1);
 		}
 	}
 }
 
+require_once($config['base_path'] . '/lib/snmp.php');
+require_once($config['base_path'] . '/lib/poller.php');
+require_once($config['base_path'] . '/lib/rrd.php');
+require_once($config['base_path'] . '/lib/ping.php');
+
+if ($version) {
+	display_version();
+	exit;
+} elseif ($help) {
+	display_help();
+	exit;
+}
+
 global $poller_db_cnn_id, $remote_db_cnn_id, $cactiphp, $using_proc_function;
+global $poller_id, $sessions, $downhosts, $print_data_to_stdout;
+
+$maxwidth   = get_max_column_width();
+
+if ($pmessage) {
+	cacti_log('Forcing poller to ' . $value, true, 'POLLER', POLLER_VERBOSITY_HIGH);
+}
 
 if ($config['poller_id'] > 1 && $config['connection'] == 'online') {
 	$poller_db_cnn_id = $remote_db_cnn_id;
@@ -166,7 +170,7 @@ $exists = db_fetch_cell_prepared('SELECT COUNT(*)
 	WHERE poller_id = ?',
 	array($poller_id));
 
-if (empty($exists)) {
+if ($exists == 0 && $poller_id > 1) {
 	record_cmdphp_done();
 	db_close();
 	exit(-1);
@@ -181,6 +185,7 @@ if (function_exists('pcntl_signal')) {
 // record the start time
 $start = microtime(true);
 $poller_interval = read_config_option('poller_interval');
+$cron_interval   = read_config_option('cron_interval');
 $active_profiles = read_config_option('active_profiles');
 
 // check arguments
@@ -191,16 +196,21 @@ if ($allhost) {
 
 	$params1    = array($poller_id);
 	$params2    = array($poller_id, POLLER_ACTION_SCRIPT_PHP, POLLER_ACTION_SCRIPT_PHP_COUNT);
-	$params3    = array($poller_interval, $poller_interval, $poller_interval, $poller_id);
+	$params3    = array($poller_interval, $poller_interval, 0, $poller_interval, $poller_id);
 } else {
 	$sql_where0 = 'WHERE poller_id > ?';
-	$sql_where1 = ' AND h.id >= ? AND h.id <= ?';
+	$sql_where1 = ' AND ((h.id >= ? AND h.id <= ?) OR h.id IS NULL)';
 	$sql_where2 = ' AND pi.host_id >= ? AND pi.host_id <= ?';
 	$sql_where3 = ' AND pi.host_id >= ? AND pi.host_id <= ?';
 
 	$params1    = array($poller_id, $first, $last);
 	$params2    = array($poller_id, POLLER_ACTION_SCRIPT_PHP, POLLER_ACTION_SCRIPT_PHP_COUNT, $first, $last);
-	$params3    = array($poller_interval, $poller_interval, $poller_interval, $poller_id, $first, $last);
+
+	if ($cron_interval == $poller_interval) {
+		$params3    = array($poller_interval, $poller_interval, 0, $poller_interval, $poller_id, $first, $last);
+	} else {
+		$params3    = array($poller_interval, $poller_interval, $poller_interval, $poller_interval, $poller_id, $first, $last);
+	}
 }
 
 if ($debug) {
@@ -246,7 +256,7 @@ if ($active_profiles != 1) {
 
 	// setup next polling interval
 	db_execute_prepared("UPDATE poller_item AS pi
-		SET rrd_next_step = IF(rrd_step = ?, 0, IF(rrd_next_step - ? < 0, rrd_step, rrd_next_step - ?))
+		SET rrd_next_step = IF(rrd_step = ?, 0, IF(rrd_next_step - ? < 0, rrd_step - ?, rrd_next_step - ?))
 		WHERE poller_id = ?
 		$sql_where3",
 		$params3);
@@ -291,7 +301,15 @@ if (cacti_sizeof($poller_items) && read_config_option('poller_enabled') == 'on')
 			2 => array('pipe', 'w')  // stderr is a pipe to write to
 		);
 
-		$cactiphp = proc_open(read_config_option('path_php_binary') . ' -q ' . $config['base_path'] . '/script_server.php cmd', $cactides, $pipes);
+		$php_path = read_config_option('path_php_binary');
+
+		$command = $php_path     . ' -q ' .
+			$config['base_path'] . '/script_server.php ' .
+			' --environ=cmd '    .
+			' --poller='         . $poller_id .
+			' --mode='           . $config['connection'];
+
+		$cactiphp = proc_open($command, $cactides, $pipes);
 		$output = fgets($pipes[1], 1024);
 
 		if (substr_count($output, 'Started') != 0) {
@@ -317,6 +335,23 @@ if (cacti_sizeof($poller_items) && read_config_option('poller_enabled') == 'on')
 			if ($last_host != '') {
 				$host_end = microtime(true);
 
+				if ($output_count > 0) {
+					cacti_log("Device[$last_host] Writing $output_count items to Poller Output Table", $print_data_to_stdout, 'POLLER', debug_level($host_id, POLLER_VERBOSITY_MEDIUM));
+
+					db_execute('INSERT IGNORE INTO poller_output
+						(local_data_id, rrd_name, time, output)
+						VALUES ' . implode(', ', $output_array), true, $poller_db_cnn_id);
+
+					if (read_config_option('boost_redirect') == 'on' && read_config_option('boost_rrd_update_enable') == 'on') {
+						db_execute('INSERT IGNORE INTO poller_output_boost
+							(local_data_id, rrd_name, time, output)
+							VALUES ' . implode(', ', $output_array), true, $poller_db_cnn_id);
+					}
+
+					$output_array = array();
+					$output_count = 0;
+				}
+
 				db_execute_prepared('UPDATE host
 					SET polling_time = ?
 					WHERE id = ?
@@ -334,7 +369,6 @@ if (cacti_sizeof($poller_items) && read_config_option('poller_enabled') == 'on')
 
 					$tot_errors += $errors;
 				}
-
 			}
 
 			$error_ds    = array();
@@ -369,7 +403,9 @@ if (cacti_sizeof($poller_items) && read_config_option('poller_enabled') == 'on')
 				$output_array[] = sprintf('(%d, %s, CURRENT_TIMESTAMP(), %s)', $item['local_data_id'], db_qstr($item['rrd_name']), db_qstr($output));
 			}
 
-			if ($output_count > 1000) {
+			if ($output_count > 2000) {
+				cacti_log("Device[$host_id] Writing $output_count items to Poller Output Table", $print_data_to_stdout, 'POLLER', debug_level($host_id, POLLER_VERBOSITY_MEDIUM));
+
 				db_execute('INSERT IGNORE INTO poller_output
 					(local_data_id, rrd_name, time, output)
 					VALUES ' . implode(', ', $output_array), true, $poller_db_cnn_id);
@@ -398,6 +434,21 @@ if (cacti_sizeof($poller_items) && read_config_option('poller_enabled') == 'on')
 	// Record the last hosts polling time
 	$host_end = microtime(true);
 
+	// Flush the items to the output table
+	if ($output_count > 0) {
+		cacti_log("Device[$host_id] Writing $output_count items to Poller Output Table", $print_data_to_stdout, 'POLLER', debug_level($host_id, POLLER_VERBOSITY_MEDIUM));
+
+		db_execute('INSERT IGNORE INTO poller_output
+			(local_data_id, rrd_name, time, output)
+			VALUES ' . implode(', ', $output_array), true, $poller_db_cnn_id);
+
+		if (read_config_option('boost_redirect') == 'on' && read_config_option('boost_rrd_update_enable') == 'on') {
+			db_execute('INSERT IGNORE INTO poller_output_boost
+				(local_data_id, rrd_name, time, output)
+				VALUES ' . implode(', ', $output_array), true, $poller_db_cnn_id);
+		}
+	}
+
 	db_execute_prepared('UPDATE host
 		SET polling_time = ?
 		WHERE id = ?
@@ -420,21 +471,9 @@ if (cacti_sizeof($poller_items) && read_config_option('poller_enabled') == 'on')
 		cacti_log('WARNING: Long Responses Errors[' . cacti_sizeof($width_dses) . '] DS[' . implode(', ', $width_dses) . ']', false, 'POLLER');
 	}
 
-	if ($output_count > 0) {
-		db_execute('INSERT IGNORE INTO poller_output
-			(local_data_id, rrd_name, time, output)
-			VALUES ' . implode(', ', $output_array), true, $poller_db_cnn_id);
-
-		if (read_config_option('boost_redirect') == 'on' && read_config_option('boost_rrd_update_enable') == 'on') {
-			db_execute('INSERT IGNORE INTO poller_output_boost
-				(local_data_id, rrd_name, time, output)
-				VALUES ' . implode(', ', $output_array), true, $poller_db_cnn_id);
-		}
-	}
-
 	if ($using_proc_function && $script_server_calls > 0) {
 		// close php server process
-		fwrite($pipes[0], "quit\r\n");
+		@fwrite($pipes[0], "quit\r\n");
 		fclose($pipes[0]);
 		fclose($pipes[1]);
 		fclose($pipes[2]);
@@ -569,7 +608,7 @@ function update_system_mibs($host_id) {
 					$uptimeAltFound = true;
 				} elseif ($name == 'snmp_sysUpTimeInstance' && !$uptimeAltFound) {
 					$uptime = $value;
-				} elseif (!empty($value)) {
+				} elseif ($name != 'snmp_sysUpTimeInstanceAlt' && !empty($value)) {
 					db_execute_prepared("UPDATE host SET $name = ?
 						WHERE deleted = ''
 						AND id = ?",
@@ -696,7 +735,7 @@ function collect_device_data(&$item, &$error_ds) {
 }
 
 function ping_and_reindex_check(&$item, $mibs) {
-	global $print_data_to_stdout, $sessions, $set_spike_kill, $poller_db_cnn_id, $cactiphp, $using_proc_function;
+	global $poller_id, $print_data_to_stdout, $sessions, $set_spike_kill, $poller_db_cnn_id, $pipes, $cactiphp, $using_proc_function;
 
 	$ping    = new Net_Ping;
 	$host_id = $item['host_id'];
@@ -913,27 +952,52 @@ function get_max_column_width() {
 	return min($pmax, $bmax);
 }
 
-function display_version() {
-	$version = get_cacti_version();
-	print "Cacti Legacy Host Data Collector, Version $version, " . COPYRIGHT_YEARS . "\n";
+function sig_handler($signo) {
+	switch ($signo) {
+		case SIGTERM:
+		case SIGINT:
+			cacti_log('WARNING: Cacti Poller process terminated by user', true, 'POLLER');
+
+			// record the process as having completed
+			record_cmdphp_done();
+			db_close();
+
+			exit;
+			break;
+		default:
+			// ignore all other signals
+	}
 }
 
-//	display_help - displays the usage of the function
+function display_version() {
+	$version = get_cacti_version();
+	print "Cacti Legacy Host Data Collector, Version $version, " . COPYRIGHT_YEARS . PHP_EOL;
+}
+
+/**
+ * display_help - displays the usage of the function
+ *
+ * @return (void)
+ */
 function display_help () {
 	display_version();
 
-	print "\nusage: cmd.php --first=ID --last=ID [--poller=ID] [--mibs] [--debug]\n\n";
-	print "Cacti's legacy data collector.  This data collector is called by poller.php\n";
-	print "every poller interval to gather information from devices.  It is recommended\n";
-	print "that every system deploy spine instead of cmd.php in production due to the built\n";
-	print "in scalability limits of cmd.php.\n\n";
-	print "Required\n";
-	print "    --first  - First host id in the range to collect from.\n";
-	print "    --last   - Last host id in the range to collect from.\n\n";
-	print "Optional:\n";
-	print "    --poller - The poller to run as.  Defaults to the system poller.\n";
-	print "    --mode   - The poller mode, either online, offline, or recovery.\n";
-	print "    --mibs   - Refresh all system mibs from hosts supporting snmp.\n";
-	print "    --debug  - Display verbose output during execution.\n\n";
+	print PHP_EOL;
+	print 'usage: cmd.php --first=ID --last=ID [--poller=ID] [--mibs] [--debug]' . PHP_EOL . PHP_EOL;
+
+	print 'Cacti\'s legacy data collector.  This data collector is called by poller.php' . PHP_EOL;
+	print 'every poller interval to gather information from devices.  It is recommended' . PHP_EOL;
+	print 'that every system deploy spine instead of cmd.php in production due to the built' . PHP_EOL;
+	print 'in scalability limits of cmd.php.' . PHP_EOL . PHP_EOL;
+
+	print 'Required' . PHP_EOL;
+	print '    --first  - First host id in the range to collect from.' . PHP_EOL;
+	print '    --last   - Last host id in the range to collect from.' . PHP_EOL . PHP_EOL;
+
+	print 'Optional:' . PHP_EOL;
+	print '    --poller - The poller to run as.  Defaults to the system poller.' . PHP_EOL;
+	print '    --mode   - The poller mode, either online, offline, or recovery.' . PHP_EOL;
+	print '    --mibs   - Refresh all system mibs from hosts supporting snmp.' . PHP_EOL;
+	print '    --debug  - Display verbose output during execution.' . PHP_EOL . PHP_EOL;
 }
 
