@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -223,7 +223,7 @@ function form_save() {
 					if (isset_request_var('header') && get_nfilter_request_var('header') == 'false') {
 						$header = '&header=false';
 					} else {
-						$header = '&header=fasle';
+						$header = '&header=false';
 					}
 				} else {
 					raise_message(2);
@@ -509,7 +509,7 @@ function data_query_sv_check_sequences($type, $snmp_query_graph_id, $field_name)
 		WHERE t.count > 1",
 		array($field_name, $snmp_query_graph_id));
 
-	// report any bad or duplicate sequencs to the log for reporting purposes
+	// report any bad or duplicate sequences to the log for reporting purposes
 	if ($bad_seq > 0) {
 		cacti_log('WARN: Found ' . $bad_seq . " Bad Sequences in $table Table", false, 'WEBUI', POLLER_VERBOSITY_HIGH);
 	}
