@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -136,6 +136,14 @@ function form_save() {
 		if (!isempty_request_var('host_id')) {
 			push_out_host(get_request_var('host_id'), $local_data_id);
 		}
+
+		if (empty($save['id'])) {
+			/**
+			 * Save the last time a data source was created/updated
+			 * for Caching.
+			 */
+			set_config_option('time_last_change_data_source', time());
+		}
 	}
 
 	if ((isset_request_var('save_component_data')) && (!is_error_message())) {
@@ -232,6 +240,14 @@ function form_save() {
 
 			if ($data_template_data_id) {
 				raise_message(1);
+
+				if (empty($save['id'])) {
+					/**
+					 * Save the last time a data source was created/updated
+					 * for Caching.
+					 */
+					set_config_option('time_last_change_data_source', time());
+				}
 			} else {
 				raise_message(2);
 			}
@@ -239,7 +255,7 @@ function form_save() {
 
 		if (!is_error_message()) {
 			/* if this is a new data source and a template has been selected, skip item creation this time
-			otherwise it throws off the templatate creation because of the NULL data */
+			otherwise it throws off the template creation because of the NULL data */
 			if (!isempty_request_var('local_data_id') || isempty_request_var('data_template_id')) {
 				/* if no template was set before the save, there will be only one data source item to save;
 				otherwise there might be >1 */
@@ -895,9 +911,11 @@ function ds_edit() {
 			foreach($dts as $dtid) {
 				$dtids[] = $dtid['data_template_id'];
 			}
-		}
 
-		$dtsql = 'SELECT id, name FROM data_template WHERE id IN(' . implode(',', $dtids) . ') ORDER BY name';
+			$dtsql = 'SELECT id, name FROM data_template WHERE id IN(' . implode(',', $dtids) . ') ORDER BY name';
+		} else {
+			$dtsql = 'SELECT id, name FROM data_template ORDER BY name';
+		}
 	} else {
 		$dtsql = 'SELECT id, name FROM data_template ORDER BY name';
 	}
@@ -1437,7 +1455,7 @@ function ds() {
 						<?php print __('Search');?>
 					</td>
 					<td>
-						<input type='text' class='ui-state-default ui-corner-all' id='rfilter' size='30' value='<?php print html_escape_request_var('rfilter');?>' onChange='applyFilter()'>
+						<input type='text' class='ui-state-default ui-corner-all' id='rfilter' size='55' value='<?php print html_escape_request_var('rfilter');?>' onChange='applyFilter()'>
 					</td>
 					<td>
 						<?php print __('Data Sources');?>
@@ -1511,9 +1529,9 @@ function ds() {
 	if (get_request_var('status') == '-1') {
 		/* Show all items */
 	} elseif (get_request_var('status') == '1') {
-		$sql_where1 .= ($sql_where1 != '' ? ' AND':'WHERE') . ' dtd.active="on"';
+		$sql_where1 .= ($sql_where1 != '' ? ' AND':'WHERE') . ' (dtd.active = "on" AND h.disabled = "")';
 	} elseif (get_request_var('status') == '2') {
-		$sql_where1 .= ($sql_where1 != '' ? ' AND':'WHERE') . ' dtd.active=""';
+		$sql_where1 .= ($sql_where1 != '' ? ' AND':'WHERE') . ' (dtd.active = "" OR h.disabled != "")';
 	} elseif (get_request_var('status') == '3') {
 		$sql_where1 .= ($sql_where1 != '' ? ' AND':'WHERE') . ' (dl.snmp_index = "" AND dl.snmp_query_id > 0)';
 	}
@@ -1546,7 +1564,7 @@ function ds() {
 		) AS dtr
 		ON dl.id = dtr.local_data_id";
 
-		$total_rows = db_fetch_cell("SELECT COUNT(*)
+		$sql = "SELECT COUNT(*)
 			FROM data_local AS dl
 			INNER JOIN data_template_data AS dtd
 			ON dl.id = dtd.local_data_id
@@ -1555,7 +1573,9 @@ function ds() {
 			LEFT JOIN host AS h
 			ON h.id = dl.host_id
 			$orphan_join
-			$sql_where1");
+			$sql_where1";
+
+		$total_rows = get_total_row_data($_SESSION['sess_user_id'], $sql, array(), 'data_source');
 
 		$data_sources = db_fetch_assoc("SELECT dtr.local_graph_id, dtd.local_data_id,
 			dtd.name_cache, dtd.active, dtd.rrd_step, dt.name AS data_template_name,
@@ -1572,15 +1592,17 @@ function ds() {
 			$sql_order
 			$sql_limit");
 	} else {
-		$total_rows = db_fetch_cell("SELECT COUNT(*)
+		$sql = "SELECT COUNT(*)
 			FROM data_local AS dl
 			INNER JOIN data_template_data AS dtd
 			ON dl.id=dtd.local_data_id
 			LEFT JOIN data_template AS dt
-			ON dt.id=dl.data_template_id
+			ON dt.id = dl.data_template_id
 			LEFT JOIN host AS h
 			ON h.id = dl.host_id
-			$sql_where1");
+			$sql_where1";
+
+		$total_rows = get_total_row_data($_SESSION['sess_user_id'], $sql, array(), 'data_source');
 
 		$data_sources = db_fetch_assoc("SELECT dtd.local_data_id, dtd.name_cache, dtd.active,
 			dtd.rrd_step, dt.name AS data_template_name, dl.host_id, dtd.data_source_profile_id
@@ -1588,7 +1610,7 @@ function ds() {
 			INNER JOIN data_template_data AS dtd
 			ON dl.id=dtd.local_data_id
 			LEFT JOIN data_template AS dt
-			ON dt.id=dl.data_template_id
+			ON dt.id = dl.data_template_id
 			LEFT JOIN host AS h
 			ON h.id = dl.host_id
 			$sql_where1
@@ -1609,7 +1631,7 @@ function ds() {
 			'display' => __('Data Source Name'),
 			'align'   => 'left',
 			'sort'    => 'ASC',
-			'tip'     => __('The name of this Data Source. Generally programtically generated from the Data Template definition.')
+			'tip'     => __('The name of this Data Source. Generally programmatically generated from the Data Template definition.')
 		),
 		'local_data_id' => array(
 			'display' => __('ID'),
