@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -452,7 +452,7 @@ $sizes = array(
 			}, 1000);
 		}
 
-		$(window).on('load', function() {
+		$(function() {
 			imageOptionsChanged('init');
 			myCountdown = setTimeout(function() {
 				countdown_update();
