@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -320,6 +320,9 @@ function item_edit() {
 	get_filter_request_var('data_template_id');
 	/* ==================================================== */
 
+	// This column is for Graph Templates
+	unset($struct_graph_item['data_template_id']);
+
 	validate_item_vars();
 
 	$id = (!isempty_request_var('id') ? '&id=' . get_request_var('id') : '');
@@ -422,7 +425,7 @@ function item_edit() {
 
 	$header_label = __esc('Graph Items [graph: %s]', $title);
 
-	form_start('graphs_items.php', 'greph_edit');
+	form_start('graphs_items.php', 'graph_edit');
 
 	html_start_box($header_label, '100%', true, '3', 'center', '');
 
