@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -32,6 +32,7 @@ include_once('./lib/data_query.php');
 include_once('./lib/html_tree.php');
 include_once('./lib/ping.php');
 include_once('./lib/poller.php');
+include_once('./lib/reports.php');
 include_once('./lib/snmp.php');
 include_once('./lib/template.php');
 include_once('./lib/utility.php');
@@ -43,7 +44,8 @@ $device_actions = array(
 	4 => __('Change Device Settings'),
 	5 => __('Clear Statistics'),
 	6 => __('Apply Automation Rules'),
-	7 => __('Sync to Device Template')
+	7 => __('Sync to Device Template'),
+	8 => __('Place Device on Report')
 );
 
 $device_actions = api_plugin_hook_function('device_action_array', $device_actions);
@@ -228,6 +230,8 @@ function get_site_locations() {
 		FROM host
 		WHERE site_id = ?
 		AND location LIKE ?
+		AND location != ""
+		AND location IS NOT NULL
 		ORDER BY location',
 		array($site_id, "%$term%"));
 
@@ -237,6 +241,10 @@ function get_site_locations() {
 		}
 	}
 
+	if (!cacti_sizeof($return)) {
+		$return[] = array('label' => __('None'), 'value' => '', 'id' => __('None'));
+	}
+
 	print json_encode($return);
 }
 
@@ -300,10 +308,19 @@ function form_actions() {
 				api_device_disable_devices($selected_items);
 			} elseif (get_request_var('drp_action') == '4') { // change device options
 				api_device_change_options($selected_items, $_POST);
-			} elseif (get_request_var('drp_action') == '5') { // Clear Statisitics for Selected Devices
+			} elseif (get_request_var('drp_action') == '5') { // Clear Statistics for Selected Devices
 				api_device_clear_statistics($selected_items);
 			} elseif (get_request_var('drp_action') == '7') { // sync to device template
 				api_device_sync_device_templates($selected_items);
+			} elseif (get_request_var('drp_action') == '8') { // place device on report
+				if (!reports_add_devices(get_filter_request_var('report_id'), $selected_items, get_filter_request_var('timespan'), get_filter_request_var('align'))) {
+					$name = db_fetch_cell_prepared('SELECT name
+						FROM reports
+						WHERE id = ?',
+						array(get_request_var('report_id')));
+
+					raise_message('reports_add_error', __('Unable to add some Devices to Report \'%s\'', $name), MESSAGE_LEVEL_WARN);
+				}
 			} elseif (get_request_var('drp_action') == '1') { // delete
 				if (!isset_request_var('delete_type')) {
 					set_request_var('delete_type', 2);
@@ -439,7 +456,7 @@ function form_actions() {
 			device_change_javascript();
 
 			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Change Device(s) SNMP Options') . "'>";
-		} elseif (get_request_var('drp_action') == '5') { // Clear Statisitics for Selected Devices
+		} elseif (get_request_var('drp_action') == '5') { // Clear Statistics for Selected Devices
 			print "<tr>
 				<td colspan='2' class='textArea'>
 					<p>" . __('Click \'Continue\' to clear the counters for the following Device(s).') . "</p>
@@ -494,6 +511,39 @@ function form_actions() {
 			</tr>";
 
 			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel'). "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Run Automation on Device(s)') . "'>";
+        } elseif (get_request_var('drp_action') == '8') {
+			global $alignment, $graph_timespans;
+
+			$reports = db_fetch_assoc_prepared('SELECT id, name
+				FROM reports
+				WHERE user_id = ?
+				ORDER BY name',
+				array($_SESSION['sess_user_id']));
+
+			if (cacti_sizeof($reports)) {
+				print "<tr>
+					<td class='textArea'>
+						<p>" . __('Click \'Continue\' to add the selected Graphs to the Report below.') . "</p>
+						<div class='itemlist'><ul>$host_list</ul></div>
+					</td>
+				</tr>
+				<tr><td>" . __('Report Name') . '<br>';
+				form_dropdown('report_id', $reports, 'name', 'id', '', '', '0');
+				print '</td></tr>';
+
+				print '<tr><td>' . __('Timespan') . '<br>';
+				form_dropdown('timespan', $graph_timespans, '', '', '', '', read_user_setting('default_timespan'));
+				print '</td></tr>';
+
+				print '<tr><td>' . __('Align') . '<br>';
+				form_dropdown('align', $alignment, '', '', '', '', REPORTS_ALIGN_CENTER);
+				print "</td></tr>\n";
+
+				$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Add Devices to Report') . "'>";
+            } else {
+                print "<tr><td class='even'><span class='textError'>" . __('You currently have no Reports defined.') . "</span></td></tr>\n";
+                $save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Return') . "' onClick='cactiReturnTo()'>";
+            }
 		} else {
 			$save['drp_action'] = get_request_var('drp_action');
 			$save['host_list']  = $host_list;
@@ -651,6 +701,7 @@ function host_edit() {
 			}
 
 			$repop_link = "<span class='linkMarker'>*</span><a class='hyperLink' href='" . html_escape('host.php?action=repopulate&host_id=' . $host['id']) . "'>" . __('Repopulate Poller Cache') . "</a><br>";
+			$repop_link .= "<span class='linkMarker'>*</span><a class='hyperLink' href='" . html_escape('utilities.php?poller_action=-1&action=view_poller_cache&host_id=' . $host['id'] . '&template_id=-1&filter=&rows=-1') . "'>" . __('View Poller Cache') . "</a><br>";
 		}
 	} else {
 		$_SESSION['cur_device_id'] = 0;
@@ -1130,7 +1181,7 @@ function device_javascript() {
 				$('#row_ping_timeout').hide();
 				$('#row_ping_port').hide();
 				$('#row_ping_timeout').hide();
-				$('#row_ping_retrie').hide();
+				$('#row_ping_retries').hide();
 
 				break;
 			case '1': // ping and snmp sysUptime
@@ -1418,10 +1469,9 @@ function get_device_records(&$total_rows, $rows) {
 
 	$sql_where = api_plugin_hook_function('device_sql_where', $sql_where);
 
-	$total_rows = db_fetch_cell("SELECT
-		COUNT(host.id)
-		FROM host
-		$sql_where");
+	$sql = "SELECT COUNT(host.id) FROM host $sql_where";
+
+	$total_rows = get_total_row_data($_SESSION['sess_user_id'], $sql, array(), 'device');
 
 	$poller_interval = read_config_option('poller_interval');
 
@@ -1441,7 +1491,6 @@ function get_device_records(&$total_rows, $rows) {
 		LEFT JOIN (SELECT host_id, COUNT(*) AS data_sources FROM data_local GROUP BY host_id) AS dl
 		ON host.id=dl.host_id
 		$sql_where
-		GROUP BY host.id
 		$sql_order
 		$sql_limit";
 
@@ -1524,7 +1573,10 @@ function host() {
 							<option value='-1'<?php if (get_request_var('host_template_id') == '-1') {?> selected<?php }?>><?php print __('Any');?></option>
 							<option value='0'<?php if (get_request_var('host_template_id') == '0') {?> selected<?php }?>><?php print __('None');?></option>
 							<?php
-							$host_templates = db_fetch_assoc('SELECT id, name FROM host_template ORDER BY name');
+							$host_templates = db_fetch_assoc('SELECT DISTINCT ht.id, ht.name
+								FROM host_template ht
+								JOIN host h ON h.host_template_id = ht.id
+								ORDER BY ht.name');
 
 							if (cacti_sizeof($host_templates)) {
 								foreach ($host_templates as $host_template) {
@@ -1621,7 +1673,7 @@ function host() {
 			strURL += '&poller_id=' + $('#poller_id').val();
 			strURL += '&location=' + $('#location').val();
 			strURL += '&rows=' + $('#rows').val();
-			strURL += '&filter=' + escape($('#filter').val());
+			strURL += '&filter=' + $('#filter').val();
 			strURL += '&header=false';
 			loadPageNoHeader(strURL);
 		}
@@ -1768,6 +1820,14 @@ function host() {
 			$graphs_url      = $config['url_path'] . 'graphs.php?reset=1&host_id=' . $host['id'];
 			$data_source_url = $config['url_path'] . 'data_sources.php?reset=1&host_id=' . $host['id'];
 
+			if (empty($host['graphs'])) {
+				$host['graphs'] = 0;
+			}
+
+			if (empty($host['data_sources'])) {
+				$host['data_sources'] = 0;
+			}
+
 			form_alternate_row('line' . $host['id'], true);
 			form_selectable_cell(filter_value($host['description'], get_request_var('filter'), 'host.php?action=edit&id=' . $host['id']), $host['id']);
 			form_selectable_cell(filter_value($host['hostname'], get_request_var('filter')), $host['id']);
