@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -95,6 +95,9 @@ if (isset($config['cacti_version'])) {
 	exit;
 }
 
+/* Should we allow proxy ip headers? */
+$config['proxy_headers'] = (isset($proxy_headers) ? $proxy_headers : []);
+
 /* Set the poller_id */
 if (isset($poller_id)) {
 	$config['poller_id'] = $poller_id;
@@ -197,7 +200,7 @@ if (!empty($path_csrf_secret)) {
 }
 
 /* built-in snmp support */
-if (isset($php_snmp_support) && !$php_snmp_support) {
+if ((isset($php_snmp_support) && $php_snmp_support == false) || !function_exists('snmpget')) {
 	$config['php_snmp_support'] = false;
 } else {
 	$config['php_snmp_support'] = class_exists('SNMP');
@@ -254,6 +257,22 @@ if (isset($input_whitelist)) {
 	$config['input_whitelist'] = $input_whitelist;
 }
 
+if (isset($i18n_handler)) {
+	$config['i18n_language_handler'] = $i18n_handler;
+}
+
+if (isset($i18n_force_language)) {
+	$config['i18n_force_language'] = $i18n_force_language;
+}
+
+if (isset($i18n_log)) {
+	$config['i18n_log'] = $i18n_log;
+}
+
+if (isset($i18n_text_log)) {
+	$config['i18n_text_log'] = $i18n_text_log;
+}
+
 /* include base modules */
 include_once($config['library_path'] . '/database.php');
 include_once($config['library_path'] . '/functions.php');
@@ -269,8 +288,14 @@ if ((isset($no_http_headers) && $no_http_headers == true) || in_array($filename,
 	$config['is_web'] = false;
 }
 
+$auto_start = ini_get('session.auto_start');
+if ($config['is_web'] && ($auto_start == 'On' || $auto_start == '1')) {
+	print 'FATAL: PHP settings session.auto_start NOT supported.  Disable in your php.ini file and then restart your Web Service' . PHP_EOL;
+	exit;
+}
+
 /* set poller mode */
-global $local_db_cnn_id, $remote_db_cnn_id;
+global $local_db_cnn_id, $remote_db_cnn_id, $conn_mode;
 
 $config['connection'] = 'online';
 
@@ -296,8 +321,13 @@ if ($config['poller_id'] > 1 || isset($rdatabase_hostname)) {
 	/* gather the existing cactidb version */
 	$config['cacti_db_version'] = db_fetch_cell('SELECT cacti FROM version LIMIT 1', false, $local_db_cnn_id);
 
-	// We are a remote poller also try to connect to the remote database
-	$remote_db_cnn_id = db_connect_real($rdatabase_hostname, $rdatabase_username, $rdatabase_password, $rdatabase_default, $rdatabase_type, $rdatabase_port, $database_retries, $rdatabase_ssl, $rdatabase_ssl_key, $rdatabase_ssl_cert, $rdatabase_ssl_ca);
+	/**
+	 * If we have not been forced offline by the $conn_mode global and since we are
+	 * a remote poller, let's attempt to get back online.
+	 */
+	if ($conn_mode != 'offline') {
+		$remote_db_cnn_id = db_connect_real($rdatabase_hostname, $rdatabase_username, $rdatabase_password, $rdatabase_default, $rdatabase_type, $rdatabase_port, $database_retries, $rdatabase_ssl, $rdatabase_ssl_key, $rdatabase_ssl_cert, $rdatabase_ssl_ca);
+	}
 
 	if ($config['is_web'] && is_object($remote_db_cnn_id) &&
 		$config['connection'] != 'recovery' &&
@@ -338,7 +368,7 @@ if ($config['poller_id'] > 1 || isset($rdatabase_hostname)) {
 		print $li . 'the database is running.' . $il;
 		print $li . 'the credentials in config.php are valid.' . $il;
 		print $lu . $sp;
-		if (isset($_REQUEST['display_db_errors']) & !empty($config['DATABASE_ERROR'])) {
+		if (isset($_REQUEST['display_db_errors']) && !empty($config['DATABASE_ERROR'])) {
 			print $ps . 'The following database errors occurred: ' . $ul;
 			foreach ($config['DATABASE_ERROR'] as $e) {
 				print $li . $e['Code'] . ': ' . $e['Error'] . $il;
@@ -358,6 +388,9 @@ if (!is_resource_writable($log_filename)) {
 	die('System log file is not available for writing, please enable write access' . PHP_EOL . 'Log: ' . $log_filename . PHP_EOL);
 }
 
+/* prime the most popular config settings */
+prime_common_config_settings();
+
 if ($config['poller_id'] > 1) {
 	$timezone = db_fetch_cell_prepared('SELECT timezone
 		FROM poller
@@ -369,12 +402,6 @@ if ($config['poller_id'] > 1) {
 	}
 }
 
-if (isset($cacti_db_session) && $cacti_db_session && db_table_exists('sessions')) {
-	include(dirname(__FILE__) . '/session.php');
-} else {
-	$cacti_db_session = false;
-}
-
 if (!defined('IN_CACTI_INSTALL')) {
 	set_error_handler('CactiErrorHandler');
 	register_shutdown_function('CactiShutdownHandler');
@@ -424,6 +451,12 @@ if ($config['is_web']) {
 	$config['cookie_options']     = $options;
 	$config['cacti_session_name'] = $cacti_session_name;
 
+	if (isset($cacti_db_session) && $cacti_db_session && db_table_exists('sessions') && $config['connection'] == 'online') {
+		include(dirname(__FILE__) . '/session.php');
+	} else {
+		$cacti_db_session = false;
+	}
+
 	/* we don't want these pages cached */
 	header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
 	header('X-Frame-Options: SAMEORIGIN');
@@ -465,7 +498,7 @@ if ($config['is_web']) {
 		}
 	}
 
-	/* make sure to start only only Cacti session at a time */
+	/* make sure to start only Cacti session at a time */
 	if (!isset($_SESSION['cacti_cwd'])) {
 		$_SESSION['cacti_cwd'] = $config['base_path'];
 	} else {
@@ -525,6 +558,44 @@ include_once($config['library_path'] . '/api_automation.php');
 include_once($config['include_path'] . '/csrf.php');
 
 if ($config['is_web']) {
+	/* raise a message and perform a page refresh if we've changed modes */
+	if ($config['poller_id'] > 1) {
+		if (isset($_SESSION['connection_mode'])) {
+			$previous_mode = $_SESSION['connection_mode'];
+			$reload        = false;
+
+			cacti_log('Connection: ' . $config['connection'] . ', Previous Mode: ' . $previous_mode . ', Page: ' . $_SERVER['SCRIPT_NAME'], false, 'WEBUI', POLLER_VERBOSITY_DEBUG);
+
+			if ($config['connection'] == 'online' && ($config['connection'] != $previous_mode)) {
+				$reload  = true;
+				$message = __('The Main Data Collector has returned to an Online Status');
+				$level   = MESSAGE_LEVEL_INFO;
+			} else if ($config['connection'] != 'online' && $previous_mode == 'online') {
+				$reload  = true;
+				$message = __('The Main Data Collector has gone to an Offline or Recovering Status');
+				$level   = MESSAGE_LEVEL_ERROR;
+			}
+
+			if ($reload) {
+				$_SESSION['connection_mode'] = $config['connection'];
+
+				raise_message('connection_state', $message, $level);
+
+				session_destroy();
+
+				print '<div style="display:none">cactiRemoteState</div>';
+
+				exit;
+			}
+		} else {
+			cacti_log('Connection: ' . $config['connection'] . ', Previous Mode: notset', false, 'WEBUI', POLLER_VERBOSITY_DEBUG);
+
+			$previous_mode = $config['connection'];
+
+			$_SESSION['connection_mode'] = $config['connection'];
+		}
+	}
+
 	if (isset_request_var('newtheme')) {
 		unset($_SESSION['selected_theme']);
 	}
@@ -557,5 +628,4 @@ if ($config['is_web']) {
 api_plugin_hook('config_insert');
 
 /* set config cacti_version for plugins */
-$config['cacti_version'] = CACTI_VERSION;;
-
+$config['cacti_version'] = CACTI_VERSION;
