@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -132,6 +132,11 @@ $cacti_version_codes = array(
 	'1.2.17'  => '0102',
 	'1.2.18'  => '0102',
 	'1.2.19'  => '0103',
+	'1.2.20'  => '0103',
+	'1.2.21'  => '0103',
+	'1.2.22'  => '0103',
+	'1.2.23'  => '0103',
+	'1.2.24'  => '0103',
 );
 
 $messages = array(
@@ -271,13 +276,13 @@ $messages = array(
 		'message' => __('Data Input Saved.  You must update the Data Templates referencing this Data Input Method before creating Graphs or Data Sources.'),
 		'level' => MESSAGE_LEVEL_INFO),
 	'input_save_w_ds' => array(
-		'message' => __('Data Input Saved.  You must update the Data Templates referencing this Data Input Method before the Data Collectors will start using any new or modified Data Input Input Fields.'),
+		'message' => __('Data Input Saved.  You must update the Data Templates referencing this Data Input Method before the Data Collectors will start using any new or modified Data Input - Input Fields.'),
 		'level' => MESSAGE_LEVEL_INFO),
 	'input_field_save_wo_ds' => array(
 		'message' => __('Data Input Field Saved.  You must update the Data Templates referencing this Data Input Method before creating Graphs or Data Sources.'),
 		'level' => MESSAGE_LEVEL_INFO),
 	'input_field_save_w_ds' => array(
-		'message' => __('Data Input Field Saved.  You must update the Data Templates referencing this Data Input Method before the Data Collectors will start using any new or modified Data Input Input Fields.'),
+		'message' => __('Data Input Field Saved.  You must update the Data Templates referencing this Data Input Method before the Data Collectors will start using any new or modified Data Input - Input Fields.'),
 		'level' => MESSAGE_LEVEL_INFO),
 	'clog_invalid' => array(
 		'message' => __('Log file specified is not a Cacti log or archive file.'),
@@ -420,7 +425,58 @@ $cdef_functions = array(1 =>
 	'NEGINF',
 	'NOW',
 	'TIME',
-	'LTIME'
+	'LTIME',
+	'ADDNAN',
+	'TREND',
+	'TRENDNAN',
+	'PREDICT',
+	'PREDICTSIGMA',
+	'PREDICTPERC',
+	'SQRT',
+	'ATAN',
+	'ATAN2',
+	'POW',
+	'ISINF',
+	'MINNAN',
+	'MAXNAN',
+	'DEG2RAD',
+	'RAD2DEG',
+	'ABS',
+	'REV',
+	'SMIN',
+	'SMAX',
+	'MEDIAN',
+	'STDEV',
+	'PERCENT',
+	'COUNT',
+	'STEPWIDTH',
+	'NEWDAY',
+	'NEWWEEK',
+	'NEWMONTH',
+	'NEWYEAR',
+	'DEPTH',
+	'COPY',
+	'INDEX',
+	'ROLL'
+);
+
+$phperrors = array (
+	E_ERROR => 'ERROR',
+	E_WARNING => 'WARNING',
+	E_PARSE => 'PARSE',
+	E_NOTICE => 'NOTICE',
+	E_CORE_ERROR => 'CORE_ERROR',
+	E_CORE_WARNING => 'CORE_WARNING',
+	E_COMPILE_ERROR  => 'COMPILE_ERROR',
+	E_COMPILE_WARNING => 'COMPILE_WARNING',
+	E_USER_ERROR => 'USER_ERROR',
+	E_USER_WARNING => 'USER_WARNING',
+	E_USER_NOTICE  => 'USER_NOTICE',
+	E_STRICT => 'STRICT',
+	E_RECOVERABLE_ERROR  => 'RECOVERABLE_ERROR',
+	E_DEPRECATED => 'DEPRECATED',
+	E_USER_DEPRECATED  => 'USER_DEPRECATED',
+	E_ALL => 'ALL'
 );
 
 if (cacti_version_compare(get_rrdtool_version(), '1.8.0', '>=')) {
@@ -643,7 +699,7 @@ $poller_options = array(
 	2 => 'spine'
 );
 
-if (!file_exists(read_config_option('path_spine')) || !is_executable(read_config_option('path_spine'))) {
+if (read_config_option('path_spine') != '' && (!file_exists(read_config_option('path_spine')) || !is_executable(read_config_option('path_spine')))) {
 	unset($poller_options[2]);
 }
 
@@ -688,10 +744,15 @@ $heartbeats = array(
 	20     => __('%d Seconds', 20),
 	30     => __('%d Seconds', 30),
 	40     => __('%d Seconds', 40),
-	60     => __('%d Minute',  1),
+	45     => __('%d Seconds', 45),
+	60     => __('%d Seconds', 60),
+	90     => __('%d Seconds', 90),
 	120    => __('%d Minutes', 2),
 	300    => __('%d Minutes', 5),
+	330    => __('%0.1f Minutes', 5.5),
 	600    => __('%d Minutes', 10),
+	660    => __('%d Minutes', 11),
+	900    => __('%d Minutes', 15),
 	1200   => __('%d Minutes', 20),
 	1800   => __('%d Minutes', 30),
 	2400   => __('%d Minutes', 40),
@@ -754,7 +815,7 @@ $poller_sync_intervals = array(
 );
 
 $device_threads = array(
-	1 => __('1 Thread (default)'),
+	1 => __('1 Thread'),
 	2 => __('%d Threads', 2),
 	3 => __('%d Threads', 3),
 	4 => __('%d Threads', 4),
@@ -945,6 +1006,7 @@ if ($config['poller_id'] == 1 || $config['connection'] == 'online') {
 			),
 		__('Import/Export') => array(
 			'templates_import.php' => __('Import Templates'),
+			'package_import.php'   => __('Import Packages'),
 			'templates_export.php' => __('Export Templates')
 			),
 		__('Configuration')  => array(
@@ -994,6 +1056,30 @@ $menu_glyphs = array(
 	__('Troubleshooting') => 'fa fa-bug'
 );
 
+$device_classes = array(
+	'wireless'     => __('Access Points, Controllers'),
+	'application'  => __('Application Related'),
+	'cacti'        => __('Cacti Related'),
+	'database'     => __('Database Related'),
+	'facilities'   => __('Facilities Related'),
+	'general'      => __('Generic Device'),
+	'hpc'          => __('HPC/Grid Computing'),
+	'hypervisor'   => __('Hypervisor Related'),
+	'remotemgmt'   => __('ILO, IPMI, iDrac, etc.'),
+	'license'      => __('Licensing Related'),
+	'linux'        => __('Linux Related'),
+	'loadbalancer' => __('Load Balancer'),
+	'switch'       => __('Network Switch'),
+	'router'       => __('Network Router'),
+	'firewall'     => __('Network Firewall'),
+	'storage'      => __('Storage Related'),
+	'telephony'    => __('Telco Related'),
+	'webserver'    => __('Web Server Related'),
+	'windows'      => __('Windows Related'),
+	'ups'          => __('UPS Related'),
+	'unassigned'   => __('Unassigned')
+);
+
 if ((isset($_SESSION['sess_user_id']))) {
 	if (db_table_exists('external_links')) {
 		$consoles = db_fetch_assoc('SELECT id, title, extendedstyle
@@ -1004,7 +1090,7 @@ if ((isset($_SESSION['sess_user_id']))) {
 
 		if (cacti_sizeof($consoles)) {
 			foreach ($consoles as $page) {
-				if (is_realm_allowed($page['id']+10000)) {
+				if (!$config['is_web'] || is_realm_allowed($page['id']+10000)) {
 					$menuname = (isset($page['extendedstyle']) && $page['extendedstyle'] != '' ? $page['extendedstyle'] : __('External Links'));
 					$menu[$menuname]['link.php?id=' . $page['id']] = $page['title'];
 				}
@@ -1151,7 +1237,7 @@ $user_auth_roles = array(
 );
 
 $user_auth_realm_filenames = array(
-	'about.php' => 8,
+	'about.php' => -1,
 	'cdef.php' => 14,
 	'clog.php' => 18,
 	'clog_user.php' => 19,
@@ -1183,11 +1269,13 @@ $user_auth_realm_filenames = array(
 	'step_json.php' => 26,
 	'managers.php' => 15,
 	'rrdcleaner.php' => 15,
+	'rrdcheck.php' => 15,
 	'settings.php' => 15,
 	'links.php' => 15,
 	'data_queries.php' => 13,
 	'templates_export.php' => 16,
 	'templates_import.php' => 17,
+	'package_import.php' => 17,
 	'tree.php' => 4,
 	'user_admin.php' => 1,
 	'user_domains.php' => 1,
@@ -1201,8 +1289,8 @@ $user_auth_realm_filenames = array(
 	'logout.php' => -1,
 	'auth_profile.php' => 20,
 	'auth_changepassword.php' => -1,
-	'reports_user.php' => 21,
-	'reports_admin.php' => 22,
+	'reports_user.php' => 22,
+	'reports_admin.php' => 21,
 	'automation_graph_rules.php' => 23,
 	'automation_tree_rules.php' => 23,
 	'automation_templates.php' => 23,
@@ -1215,7 +1303,8 @@ $user_auth_realm_filenames = array(
 	'aggregate_graphs.php' => 5,
 	'aggregate_items.php' => 5,
 	'spikekill.php' => 1043,
-	'permission_denied.php' => -1
+	'permission_denied.php' => -1,
+	'help.php' => -1
 );
 
 $hash_type_codes = array(
@@ -1441,6 +1530,13 @@ $dsstats_hourly_avg = array(
 	'360' => __('%d Hours', 6)
 );
 
+$rrdcheck_intervals = array(
+	'boost' => __('After Boost'),
+	'60'  => __('1 Hour'),
+	'240' => __('%d Hours', 4),
+	'1440' => __('%d Hours', 24)
+);
+
 $boost_max_rows_per_select = array(
 	'2000'   => __('2,000 Data Source Items'),
 	'5000'   => __('5,000 Data Source Items'),
@@ -1490,7 +1586,7 @@ $boost_max_memory = array(
 	'5120' => __('%d GBytes', 5),
 	'6144' => __('%d GBytes', 6),
 	'8192' => __('%d GBytes', 8),
-	'-1' => __('Infinity')
+	'-1' => __('Unlimited')
 );
 
 $realtime_window = array(
@@ -1536,7 +1632,7 @@ $reports_actions = array(
 	REPORTS_DELETE    => __('Delete'),
 );
 
-if (is_realm_allowed(22)) {
+if (!$config['is_web'] || is_realm_allowed(21)) {
 	$reports_actions[REPORTS_OWN] = __('Take Ownership');
 }
 
@@ -1569,6 +1665,7 @@ $item_types = array(
 	REPORTS_ITEM_TEXT  => __('Text'),
 	REPORTS_ITEM_TREE  => __('Tree'),
 	REPORTS_ITEM_GRAPH => __('Graph'),
+	REPORTS_ITEM_HOST  => __('Device'),
 	REPORTS_ITEM_HR    => __('Horizontal Rule')
 );
 
@@ -1774,7 +1871,7 @@ $i18n_months = array(
 	'January'	=> __('January'),
 	'February'	=> __('February'),
 	'March'		=> __('March'),
-	'Arpil'		=> __('April'),
+	'April'		=> __('April'),
 	'May'		=> __('May'),
 	'June'		=> __('June'),
 	'July'		=> __('July'),
@@ -1800,6 +1897,22 @@ $i18n_months_short = array(
 	'Dec'	=> __x('A short textual representation of a month, three letters', 'Dec'),
 );
 
+$i18n_supported_languages = array(
+	CACTI_LANGUAGE_HANDLER_DEFAULT => __('Use the First Handler Found'),
+);
+
+if (is_dir($config['base_path'] . '/include/vendor/phpgettext')) {
+	$i18n_supported_languages[CACTI_LANGUAGE_HANDLER_PHPGETTEXT]  = __('Use the PHP GetText Handler');
+}
+
+if (is_dir($config['base_path'] . '/include/vendor/gettext') && version_compare(PHP_VERSION, '8.0', '<=')) {
+	$i18n_supported_languages[CACTI_LANGUAGE_HANDLER_OSCAROTERO]  = __('Use the Oscarotero GetText Handler');
+}
+
+if (is_dir($config['base_path'] . '/include/vendor/monotranslator')) {
+	$i18n_supported_languages[CACTI_LANGUAGE_HANDLER_MOTRANSLATOR]  = __('Use the MonoTranslator GetText Handler');
+}
+
 $i18n_weekdays = array(
 	'Sunday'	=> __('Sunday'),
 	'Monday'	=> __('Monday'),
@@ -1857,27 +1970,9 @@ $i18n_themes = array(
 $database_statuses = array(
 	0 => __('[Fail]'),
 	1 => __('[Warning]'),
-	2 => __('[Success]'),
-	3 => __('[Skipped]'),
-);
-
-$phperrors = array (
-	E_ERROR => 'ERROR',
-	E_WARNING => 'WARNING',
-	E_PARSE => 'PARSE',
-	E_NOTICE => 'NOTICE',
-	E_CORE_ERROR => 'CORE_ERROR',
-	E_CORE_WARNING => 'CORE_WARNING',
-	E_COMPILE_ERROR  => 'COMPILE_ERROR',
-	E_COMPILE_WARNING => 'COMPILE_WARNING',
-	E_USER_ERROR => 'USER_ERROR',
-	E_USER_WARNING => 'USER_WARNING',
-	E_USER_NOTICE  => 'USER_NOTICE',
-	E_STRICT => 'STRICT',
-	E_RECOVERABLE_ERROR  => 'RECOVERABLE_ERROR',
-	E_DEPRECATED => 'DEPRECATED',
-	E_USER_DEPRECATED  => 'USER_DEPRECATED',
-	E_ALL => 'ALL'
+	2 => __('[Restart]'),
+	3 => __('[Success]'),
+	4 => __('[Skipped]'),
 );
 
 $navigation = array(
@@ -2081,6 +2176,30 @@ $navigation = array(
 		'url' => '',
 		'level' => '2'
 	),
+	'pollers.php:' => array(
+		'title' => __('Data Collectors'),
+		'mapping' => 'index.php:',
+		'url' => 'pollers.php',
+		'level' => '1'
+	),
+	'pollers.php:edit' => array(
+		'title' => __('(Edit)'),
+		'mapping' => 'index.php:,pollers.php:',
+		'url' => '',
+		'level' => '2'
+	),
+	'links.php:' => array(
+		'title' => __('External Links'),
+		'mapping' => 'index.php:',
+		'url' => 'links.php',
+		'level' => '1'
+	),
+	'links.php:edit' => array(
+		'title' => __('(Edit)'),
+		'mapping' => 'index.php:,links.php:',
+		'url' => '',
+		'level' => '2'
+	),
 	'color.php:' => array(
 		'title' => __('Colors'),
 		'mapping' => 'index.php:',
@@ -2303,6 +2422,12 @@ $navigation = array(
 		'url' => 'rrdcleaner.php?action=restart',
 		'level' => '2'
 	),
+	'rrdcheck.php:' => array(
+		'title' => __('RRD Check'),
+		'mapping' => 'index.php:,utilities.php:',
+		'url' => 'rrdcheck.php',
+		'level' => '2'
+	),
 	'utilities.php:' => array(
 		'title' => __('Utilities'),
 		'mapping' => 'index.php:',
@@ -2456,7 +2581,7 @@ $navigation = array(
 	'user_domains.php:edit' => array(
 		'title' => __('(Edit)'),
 		'mapping' => 'user_domains.php:,index.php:',
-		'url' => 'user_domains.php:edit',
+		'url' => 'user_domains.php',
 		'level' => '2'
 	),
 	'user_group_admin.php:' => array(
@@ -2479,10 +2604,16 @@ $navigation = array(
 	),
 	'about.php:' => array(
 		'title' => __('About Cacti'),
-		'mapping' => 'index.php:',
+		'mapping' => '',
 		'url' => 'about.php',
 		'level' => '1'
 	),
+	'help.php:' => array(
+		'title' => __('Cacti Help'),
+		'mapping' => '',
+		'url' => 'help.php',
+		'level' => '1'
+	),
 	'templates_export.php:' => array(
 		'title' => __('Export Templates'),
 		'mapping' => 'index.php:',
@@ -2501,6 +2632,12 @@ $navigation = array(
 		'url' => 'templates_import.php',
 		'level' => '1'
 	),
+	'package_import.php:' => array(
+		'title' => __('Import Packages'),
+		'mapping' => 'index.php:',
+		'url' => 'package_import.php',
+		'level' => '1'
+	),
 	'reports_admin.php:' => array(
 		'title' => __('Reporting'),
 		'mapping' => '',
@@ -2754,4 +2891,34 @@ $graph_sources = array(
 	3 => __('Aggregate'),
 );
 
+if ($config['cacti_server_os'] == 'unix') {
+	$dejavu_paths = array(
+		'/usr/share/fonts/dejavu/', //RHEL/CentOS
+		'/usr/share/fonts/truetype/', //SLES
+		'/usr/share/fonts/truetype/dejavu/', //Ubuntu
+		'/usr/local/share/fonts/dejavu/', //FreeBSD
+		__DIR__ . '/fonts'  //Build-in
+	);
+} else {
+	$dejavu_paths = array(
+		'C:/Windows/Fonts/' //Windows
+	);
+}
+
+$allowed_proxy_headers =	array(
+	'X-Forwarded-For',
+	'X-Client-IP',
+	'X-Real-IP',
+	'X-ProxyUser-Ip',
+	'CF-Connecting-IP',
+	'True-Client-IP',
+	'HTTP_X_FORWARDED',
+	'HTTP_X_FORWARDED_FOR',
+	'HTTP_X_CLUSTER_CLIENT_IP',
+	'HTTP_FORWARDED_FOR',
+	'HTTP_FORWARDED',
+	'HTTP_CLIENT_IP',
+	'REMOTE_ADDR',
+);
+
 api_plugin_hook('config_arrays');
