@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -88,6 +88,8 @@ function api_delete_graphs(&$local_graph_ids, $delete_type) {
 			if (cacti_sizeof($data_sources)) {
 				api_data_source_remove_multi($data_sources);
 			}
+		} else {
+			api_graph_remove_multi($local_graph_ids);
 		}
 
 		break;
@@ -96,6 +98,12 @@ function api_delete_graphs(&$local_graph_ids, $delete_type) {
 
 		break;
 	}
+
+	/**
+	 * Save the last time a graph was created/updated
+	 * for Caching.
+	 */
+	set_config_option('time_last_change_graph', time());
 }
 
 function api_graph_remove($local_graph_id) {
@@ -114,6 +122,12 @@ function api_graph_remove($local_graph_id) {
 	db_execute_prepared('DELETE FROM graph_tree_items WHERE local_graph_id = ?', array($local_graph_id));
 	db_execute_prepared('DELETE FROM reports_items WHERE local_graph_id = ?', array($local_graph_id));
 	db_execute_prepared('DELETE FROM graph_local WHERE id = ?', array($local_graph_id));
+
+	/**
+	 * Save the last time a graph was created/updated
+	 * for Caching.
+	 */
+	set_config_option('time_last_change_graph', time());
 }
 
 function api_graph_remove_bad_graphs(&$local_graph_ids = array()) {
@@ -134,6 +148,12 @@ function api_graph_remove_bad_graphs(&$local_graph_ids = array()) {
 				AND local_graph_id = 0');
 		}
 	}
+
+	/**
+	 * Save the last time a graph was created/updated
+	 * for Caching.
+	 */
+	set_config_option('time_last_change_graph', time());
 }
 
 function api_graph_remove_aggregate_items($local_graph_ids) {
@@ -206,6 +226,12 @@ function api_graph_remove_multi($local_graph_ids) {
 			db_execute("DELETE FROM reports_items WHERE local_graph_id IN ($ids_to_delete)");
 			db_execute("DELETE FROM graph_local WHERE id IN ($ids_to_delete)");
 		}
+
+		/**
+		 * Save the last time a graph was created/updated
+		 * for Caching.
+		 */
+		set_config_option('time_last_change_graph', time());
 	}
 }
 
@@ -473,7 +499,7 @@ function api_duplicate_graph($_local_graph_id, $_graph_template_id, $graph_title
 				unset($save);
 
 				$save['id']                = 0;
-				$save['hash']              = get_hash_data_query('', 'data_query_graph');
+				$save['hash']              = get_hash_data_query(0, 'data_query_graph');
 				$save['snmp_query_id']     = $dqg['snmp_query_id'];
 				$save['name']              = $name;
 				$save['graph_template_id'] = $graph_template_id;
@@ -548,6 +574,12 @@ function api_duplicate_graph($_local_graph_id, $_graph_template_id, $graph_title
 			}
 		}
 	}
+
+	/**
+	 * Save the last time a graph was created/updated
+	 * for Caching.
+	 */
+	set_config_option('time_last_change_graph', time());
 }
 
 function api_graph_change_device($local_graph_id, $host_id) {
