@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -38,12 +38,13 @@
  * @param  (string) String that points to the client ssl cert file
  * @param  (string) String that points to the ssl ca file
  *
- * @returns (bool) '1' for success, false for error
+ * @returns (bool|object) connection object on success, false for error
  */
 function db_connect_real($device, $user, $pass, $db_name, $db_type = 'mysql', $port = '3306', $retries = 20,
 	$db_ssl = false, $db_ssl_key = '', $db_ssl_cert = '', $db_ssl_ca = '', $persist = false) {
 
-	global $database_sessions, $database_total_queries, $database_persist, $config;
+	global $database_sessions, $database_details, $database_total_queries, $database_persist, $config;
+
 	$database_total_queries = 0;
 
 	$i = 0;
@@ -94,6 +95,10 @@ function db_connect_real($device, $user, $pass, $db_name, $db_type = 'mysql', $p
 		}
 	}
 
+	/* set connection timout for down servers */
+	$flags[PDO::ATTR_TIMEOUT] = 2;
+	$flage[PDO::ATTR_ERRMODE] = PDO::ERRMODE_EXCEPTION;
+
 	while ($i <= $retries) {
 		try {
 			if (strpos($device, '/') !== false && filetype($device) == 'socket') {
@@ -115,6 +120,24 @@ function db_connect_real($device, $user, $pass, $db_name, $db_type = 'mysql', $p
 
 			$database_sessions["$odevice:$port:$db_name"] = $cnn_id;
 
+			$object_hash = spl_object_hash($cnn_id);
+
+			$database_details[$object_hash] = array(
+				'database_conn'     => $cnn_id,
+				'database_hostname' => $device,
+				'database_username' => $user,
+				'database_password' => $pass,
+				'database_default'  => $db_name,
+				'database_type'     => $db_type,
+				'database_port'     => $port,
+				'database_retries'  => $retries,
+				'database_ssl'      => $db_ssl,
+				'database_ssl_key'  => $db_ssl_key,
+				'database_ssl_cert' => $db_ssl_cert,
+				'database_ssl_ca'   => $db_ssl_ca,
+				'database_persist'  => $persist,
+			);
+
 			$ver = db_get_global_variable('version', $cnn_id);
 
 			if (strpos($ver, 'MariaDB') !== false) {
@@ -199,6 +222,66 @@ function db_connect_real($device, $user, $pass, $db_name, $db_type = 'mysql', $p
 	return false;
 }
 
+function db_check_reconnect($db_conn = false) {
+	global $config, $database_details;
+
+	include($config['base_path'] . '/include/config.php');
+
+	if (cacti_sizeof($database_details) && $db_conn !== false) {
+		foreach($database_details as $det) {
+			if (spl_object_hash($det['database_conn']) == spl_object_hash($db_conn)) {
+				$database_hostname = $det['database_hostname'];
+				$database_username = $det['database_username'];
+				$database_password = $det['database_password'];
+				$database_default  = $det['database_default'];
+				$database_type     = $det['database_type'];
+				$database_port     = $det['database_port'];
+				$database_retries  = $det['database_retries'];
+				$database_ssl      = $det['database_ssl'];
+				$database_ssl_key  = $det['database_ssl_key'];
+				$database_ssl_cert = $det['database_ssl_cert'];
+				$database_ssl_ca   = $det['database_ssl_ca'];
+
+				break;
+			}
+		}
+	} else {
+		if (!isset($database_ssl))      $database_ssl      = false;
+		if (!isset($database_ssl_key))  $database_ssl_key  = '';
+		if (!isset($database_ssl_cert)) $database_ssl_cert = '';
+		if (!isset($database_ssl_ca))   $database_ssl_ca   = '';
+		if (!isset($database_retries))  $database_retries  = 2;
+		if (!isset($database_port))     $database_port     = 3306;
+	}
+
+	if ($db_conn !== false) {
+		$version = db_fetch_cell('SELECT 1', '', false, $db_conn);
+	} else {
+		$version = db_fetch_cell('SELECT 1');
+	}
+
+	if ($version === false) {
+		syslog(LOG_ALERT, 'CACTI: Database Connection went away.  Attempting to reconnect!');
+
+		db_close();
+
+		// Connect to the database server
+		db_connect_real(
+			$database_hostname,
+			$database_username,
+			$database_password,
+			$database_default,
+			$database_type,
+			$database_port,
+			$database_retries,
+			$database_ssl,
+			$database_ssl_key,
+			$database_ssl_cert,
+			$database_ssl_ca
+		);
+	}
+}
+
 function db_warning_handler($errno, $errstr, $errfile, $errline, $errcontext = []) {
 	throw new Exception($errstr, $errno);
 }
@@ -264,7 +347,7 @@ function db_close($db_conn = false) {
  *
  * @param  (string)        The SQL query to execute
  * @param  (bool)          Whether to log error messages, defaults to true
- * @param  (bool|resource) The connection to use or false for the defaul
+ * @param  (bool|resource) The connection to use or false for the default
  *
  * @return (bool) '1' for success, false on error
  */
@@ -287,7 +370,8 @@ function db_execute($sql, $log = true, $db_conn = false) {
  * @return (bool) '1' for success, false for failed
  */
 function db_execute_prepared($sql, $params = array(), $log = true, $db_conn = false, $execute_name = 'Exec', $default_value = true, $return_func = 'no_return_function', $return_params = array()) {
-	global $database_sessions, $database_default, $config, $database_hostname, $database_port, $database_total_queries, $database_last_error, $database_log;
+	global $database_sessions, $database_default, $config, $database_hostname, $database_port, $database_total_queries, $database_last_error, $database_log, $affected_rows;
+
 	$database_total_queries++;
 
 	if (!isset($database_log)) {
@@ -313,7 +397,8 @@ function db_execute_prepared($sql, $params = array(), $log = true, $db_conn = fa
 	}
 
 	$errors = 0;
-	$db_conn->affected_rows = 0;
+
+	$affected_rows[spl_object_hash($db_conn)] = 0;
 
 	while (true) {
 		$query = $db_conn->prepare($sql);
@@ -358,8 +443,7 @@ function db_execute_prepared($sql, $params = array(), $log = true, $db_conn = fa
 		}
 
 		if ($en == '') {
-			// With PDO, we have to free this up
-			$db_conn->affected_rows = $query->rowCount();
+			$affected_rows[spl_object_hash($db_conn)] = $query->rowCount();
 
 			$return_value = $default_value;
 			if (function_exists($return_func)) {
@@ -404,7 +488,7 @@ function db_execute_prepared($sql, $params = array(), $log = true, $db_conn = fa
 
 						continue;
 					}
-				} else if ($en == 1153) {
+				} elseif ($en == 1153) {
 					if (strlen($sql) > 1024) {
 						$sql = substr($sql, 0, 1024) . '...';
 					}
@@ -633,7 +717,7 @@ function db_fetch_assoc_return($query) {
 }
 
 /**
- * db_fetch_insert_id - get the last insert_id or auto incriment
+ * db_fetch_insert_id - get the last insert_id or auto increment
  *
  * @param  (bool|resource) The connection to use or false to use the default
  *
@@ -663,7 +747,7 @@ function db_fetch_insert_id($db_conn = false) {
  *                         or false on error
  */
 function db_affected_rows($db_conn = false) {
-	global $database_sessions, $database_default, $database_hostname, $database_port;
+	global $database_sessions, $database_default, $database_hostname, $database_port, $affected_rows;
 
 	/* check for a connection being passed, if not use legacy behavior */
 	if (!is_object($db_conn)) {
@@ -674,7 +758,7 @@ function db_affected_rows($db_conn = false) {
 		}
 	}
 
-	return $db_conn->affected_rows;
+	return $affected_rows[spl_object_hash($db_conn)];
 }
 
 /**
@@ -976,8 +1060,8 @@ function db_cacti_initialized($is_web = true) {
 		print ($is_web ? '<table style="height:40px;"><tr><td></td></tr></table>':'');
 		print ($is_web ? '<table style="margin-left:auto;margin-right:auto;width:80%;border:1px solid rgba(98,125,77,1)" class="cactiTable"><tr class="cactiTableTitle"><td style="color:snow;font-weight:bold;">Fatal Error - Cacti Database Not Initialized</td></tr>':'');
 		print ($is_web ? '<tr class="installArea"><td>':'');
-		print ($is_web ? '<p>':'') . 'The Cacti Database has not been initialized.  Please initilize it before continuing.' . ($is_web ? '</p>':"\n");
-		print ($is_web ? '<p>':'') . 'To initilize the Cacti database, issue the following commands either as root or using a valid account.' . ($is_web ? '</p>':"\n");
+		print ($is_web ? '<p>':'') . 'The Cacti Database has not been initialized.  Please initialize it before continuing.' . ($is_web ? '</p>':"\n");
+		print ($is_web ? '<p>':'') . 'To initialize the Cacti database, issue the following commands either as root or using a valid account.' . ($is_web ? '</p>':"\n");
 		print ($is_web ? '<p style="font-weight:bold;padding-left:25px;">':'') . '  mysqladmin -uroot -p create cacti' . ($is_web ? '</p>':"\n");
 		print ($is_web ? '<p style="font-weight:bold;padding-left:25px;">':'') . '  mysql -uroot -p -e "grant all on cacti.* to \'someuser\'@\'localhost\' identified by \'somepassword\'"' . ($is_web ? '</p>':"\n");
 		print ($is_web ? '<p style="font-weight:bold;padding-left:25px;">':'') . '  mysql -uroot -p -e "grant select on mysql.time_zone_name to \'someuser\'@\'localhost\' identified by \'somepassword\'"' . ($is_web ? '</p>':"\n");
@@ -1044,14 +1128,14 @@ function db_get_table_column_types($table, $db_conn = false) {
 }
 
 /**
- * db_update_table - a function that will update the table strucutre based upon
+ * db_update_table - a function that will update the table structure based upon
  *   a Cacti specific array specification constructed by the sqltable_to_php.php
  *   script.  That script will construct an array from the table definition.
  *   The script is very handy for both Cacti table construction and for plugins.
  *
  * @param  (string)        The name of the table
  * @param  (array)         Table definition as a Cacti specific array
- * @param  (bool)          Remove any existing columns that are not in the speicfication
+ * @param  (bool)          Remove any existing columns that are not in the specification
  * @param  (bool)          Whether to log error messages, defaults to true
  * @param  (bool|resource) The connection to use or false to use the default
  *
@@ -1094,7 +1178,7 @@ function db_update_table($table, $data, $removecolumns = false, $log = true, $db
 		}
 	}
 
-	if (isset($data['row_format']) && db_get_global_variable('innodb_file_format', $db_conn) == 'Barracuda') {
+	if (isset($data['row_format']) && strtolower(db_get_global_variable('innodb_file_format', $db_conn)) == 'barracuda') {
 		db_execute("ALTER TABLE `$table` ROW_FORMAT = " . $data['row_format'], $log, $db_conn);
 	}
 
@@ -1257,7 +1341,7 @@ function db_update_table($table, $data, $removecolumns = false, $log = true, $db
 }
 
 /**
- * db_format_index_create - Converts and array of indexs to a string
+ * db_format_index_create - Converts and array of indexes to a string
  *   that is compatible with the cacti database table creation array.
  *
  * @param  (array) An array of indexes to process
@@ -1386,7 +1470,7 @@ function db_table_create($table, $data, $log = true, $db_conn = false) {
 			$sql .= " COMMENT = '" . $data['comment'] . "'";
 		}
 
-		if (isset($data['row_format']) && db_get_global_variable('innodb_file_format', $db_conn) == 'Barracuda') {
+		if (isset($data['row_format']) && strtolower(db_get_global_variable('innodb_file_format', $db_conn)) == 'barracuda') {
 			$sql .= ' ROW_FORMAT = ' . $data['row_format'];
 		}
 
@@ -1532,7 +1616,7 @@ function db_rollback_transaction($db_conn = false) {
 }
 
 /**
- * array_to_sql_or - loops through a single dimentional array and converts each
+ * array_to_sql_or - loops through a single dimensional array and converts each
  *   item to a string that can be used in the OR portion of an sql query in the
  *   following form:
  *
@@ -1564,7 +1648,7 @@ function array_to_sql_or($array, $sql_column) {
  * @param $keyCols - a string or array of primary keys
  * @param $autoQuote - whether to use intelligent quoting or not
  *
- * @return - the auto incriment id column (if applicable)
+ * @return - the auto increment id column (if applicable)
  */
 function db_replace($table_name, $array_items, $keyCols, $db_conn = false) {
 	global $database_sessions, $database_default, $database_hostname, $database_port;
@@ -1586,7 +1670,7 @@ function db_replace($table_name, $array_items, $keyCols, $db_conn = false) {
  *
  * @param  (resource)     The database connection to use
  * @param  (string)       The table name to use
- * @param  (array)        An array of fieldvalues
+ * @param  (array)        An array of field values
  * @param  (string|array) A string of a key column or an array of key columns
  *
  * @return (bool|int) Either the insert id of the replace of false on error
@@ -1651,7 +1735,7 @@ function _db_replace($db_conn, $table, $fieldArray, $keyCols) {
  * @param  (string)       The name of the table to make the replacement in
  * @param  (string|array) The primary key(s) for the table
  *
- * @return (bool|int)     The auto incriment id column (if applicable)
+ * @return (bool|int)     The auto increment id column (if applicable)
  */
 function sql_save($array_items, $table_name, $key_cols = 'id', $autoinc = true, $db_conn = false) {
 	global $database_sessions, $database_default, $database_hostname, $database_port, $database_last_error;
@@ -1727,7 +1811,7 @@ function sql_save($array_items, $table_name, $key_cols = 'id', $autoinc = true,
 
 /**
  * db_qstr - Quote a string using the PDO function and also enclose
- *   the ramainder of the string in single quotes.
+ *   the remainder of the string in single quotes.
  *
  * @param  (string)        The SQL to be escaped
  * @param  (bool|resource) The database connection or false if to use the default
@@ -1849,7 +1933,7 @@ function db_check_password_length() {
 
 	if ($len === false) {
 		die(__('Failed to determine password field length, can not continue as may corrupt password'));
-	} else if ($len < 80) {
+	} elseif ($len < 80) {
 		/* Ensure that the password length is increased before we start updating it */
 		db_execute("ALTER TABLE user_auth MODIFY COLUMN password varchar(256) NOT NULL default ''");
 
@@ -1905,7 +1989,7 @@ function db_get_default_database($db_conn = false) {
 /**
  * db_force_remote_cnn - alias for db_switch_remote_to_main()
  *
- * Swithces the local database connection to the main server
+ * Switches the local database connection to the main server
  * This is required for CLI script that wish to talk to the main
  * database server since by default they are connected to the local
  * database server.
@@ -1922,10 +2006,10 @@ function db_force_remote_cnn() {
  * This function needs to be used with caution.  It is for switching a database connection
  * from the remote connection or the main Cacti poller back to the local connection
  * for all db* calls that do not require the connection to be passed.  It's to be used
- * by CLI script, that by default connecto the local database, back and forth to the
+ * by CLI script, that by default connect to the local database, back and forth to the
  * remote or main database server.
  *
- * @returns (bool) If the switch was succefull
+ * @returns (bool) If the switch was successful
  */
 function db_switch_remote_to_main() {
     global $config, $database_sessions, $database_hostname, $database_port, $database_default;
@@ -1946,10 +2030,10 @@ function db_switch_remote_to_main() {
  * This function needs to be used with caution.  It is for switching a database connection
  * from the remote connection or the main Cacti poller back to the local connection
  * for all db* calls that do not require the connection to be passed.  It's to be used
- * by CLI script, that by default connecto the local database, back and forth to the
+ * by CLI script, that by default connect to the local database, back and forth to the
  * remote or main database server.
  *
- * @returns (bool) If the switch was succefull
+ * @returns (bool) If the switch was successful
  */
 function db_switch_main_to_local() {
     global $config, $database_sessions, $database_hostname, $database_port, $database_default;
@@ -1988,15 +2072,15 @@ function db_dump_data($database = '', $tables = '', $credentials = array(), $out
 			if (strstr($name, '--') !== false) {      //name like --host
 				if($name == '--password') {
 					$password = $value;
-				} else if ($name == '--user') {
+				} elseif ($name == '--user') {
 					$username = $value;
 				} else {
 					$credentials_string .= $name . '=' . $value . ' ';
 				}
-			} else if(strstr($name, '-') !== false) { //name like -h
+			} elseif(strstr($name, '-') !== false) { //name like -h
 				if($name == '-p') {
 					$password = $value;
-				} else if ($name == '-u') {
+				} elseif ($name == '-u') {
 					$username = $value;
 				} else {
 					$credentials_string .= $name . $value . ' ';
@@ -2004,7 +2088,7 @@ function db_dump_data($database = '', $tables = '', $credentials = array(), $out
 			} else {                                  //name like host
 				if($name == 'password') {
 					$password = $value;
-				} else if ($name == 'user') {
+				} elseif ($name == 'user') {
 					$username = $value;
 				} else {
 					$credentials_string .= '--' . $name . '=' . $value . ' ';
