@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -111,7 +111,7 @@ function cacti_ldap_auth($username, $password = '', $dn = '', $host = '', $port
   @arg $version - '2' or '3', LDAP protocol version, Default = Configured settings value
   @arg $encryption - '0' None, '1' SSL, '2' TLS, Default = Configured settings value
   @arg $referrals - '0' Referrals from server are ignored, '1' Referrals from server are processed, Default = Configured setting value
-  @arg $mode - '0' No Searching, '1' Anonymous Searching, '2' Specfic Searching, Default = Configured settings value
+  @arg $mode - '0' No Searching, '1' Anonymous Searching, '2' Specific Searching, Default = Configured settings value
   @arg $search_base - Search base DN, Default = Configured settings value
   @arg $search_filter - Filter to find the user, Default = Configured settings value
   @arg $specific_dn - DN for binding to perform user search, Default = Configured settings value
@@ -192,7 +192,7 @@ function cacti_ldap_search_dn($username, $dn = '', $host = '', $port = '', $port
   @arg $version - '2' or '3', LDAP protocol version, Default = Configured settings value
   @arg $encryption - '0' None, '1' SSL, '2' TLS, Default = Configured settings value
   @arg $referrals - '0' Referrals from server are ignored, '1' Referrals from server are processed, Default = Configured setting value
-  @arg $mode - '0' No Searching, '1' Anonymous Searching, '2' Specfic Searching, Default = Configured settings value
+  @arg $mode - '0' No Searching, '1' Anonymous Searching, '2' Specific Searching, Default = Configured settings value
   @arg $search_base - Search base DN, Default = Configured settings value
   @arg $search_filter - Filter to find the user, Default = Configured settings value
   @arg $specific_dn - DN for binding to perform user search, Default = Configured settings value
@@ -265,6 +265,7 @@ abstract class LdapError {
 	const SearchFoundNoUser     = 14;
 	const SearchFoundNoUserDN   = 15;
 	const UndefinedDnOrPassword = 16;
+	const EmptyPassword         = 17;
 	const Disabled              = 99;
 
 	public static function GetErrorDetails($returnError, $ldapConn = null, $ldapServer = '', $ldapError = 0) {
@@ -349,6 +350,10 @@ abstract class LdapError {
 				$error_text = __('Specific DN and Password required');
 				break;
 
+			case LdapError::EmptyPassword:
+				$error_text = __('Invalid Password provided.  Login failed.');
+				break;
+
 			default:
 				$error_text = __('Unexpected error %s (Ldap Error: %s) on Server (%s)', $returnError, $ldapError, $ldapServer);
 				break;
@@ -365,6 +370,26 @@ abstract class LdapError {
 }
 
 class Ldap {
+	public $dn;
+	public $host;
+	public $username;
+	public $password;
+	public $port;
+	public $port_ssl;
+	public $version;
+	public $encryption;
+	public $referrals;
+	public $debug;
+	public $group_require;
+	public $group_dn;
+	public $group_attrib;
+	public $group_member_type;
+	public $mode;
+	public $search_base;
+	public $search_filter;
+	public $specific_dn;
+	public $specific_password;
+
 	function __construct() {
 		/* Initialize LDAP parameters for Authenticate */
 		$this->dn         = read_config_option('ldap_dn');
@@ -374,6 +399,13 @@ class Ldap {
 		$this->version    = read_config_option('ldap_version');
 		$this->encryption = read_config_option('ldap_encryption');
 		$this->referrals  = read_config_option('ldap_referrals');
+		$this->debug      = read_config_option('ldap_debug');
+
+		if ($this->debug == 'on') {
+			$this->debug = POLLER_VERBOSITY_LOW;
+		} else {
+			$this->debug = POLLER_VERBOSITY_HIGH;
+		}
 
 		if (read_config_option('ldap_group_require') == 'on') {
 			$this->group_require = true;
@@ -426,7 +458,7 @@ class Ldap {
 	function RecordError($output, $section = 'LDAP') {
 		$logDN = empty($output['dn']) ? '' : (', DN: ' . $output['dn']);
 		cacti_log($section . ': ' . $output['error_text'] . $logDN, false, 'AUTH');
-		cacti_log($section . ': ' . $output['stack'], false, 'AUTH', POLLER_VERBOSITY_HIGH);
+		cacti_log($section . ': ' . $output['stack'], false, 'AUTH', $this->debug);
 	}
 
 	function Connect() {
@@ -455,27 +487,27 @@ class Ldap {
 
 		/* Set debug if selective debug is enabled.  This places log data into the apache error_log */
 		if (get_selective_log_level() == POLLER_VERBOSITY_DEBUG) {
-			cacti_log('LDAP: Setting php-ldap into DEBUG mode.  Check your Web Server error_log for details', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('LDAP: Setting php-ldap into DEBUG mode.  Check your Web Server error_log for details', false, 'AUTH', $this->debug);
 			ldap_set_option(null, LDAP_OPT_DEBUG_LEVEL, 7);
 		}
 
 		if (getenv('TLS_CERT') != '' && defined('LDAP_OPT_X_TLS_CERTFILE')) {
-			cacti_log('NOTE: Settings TLS_CERT to ' . getenv('TLS_CERT'), false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('NOTE: Settings TLS_CERT to ' . getenv('TLS_CERT'), false, 'AUTH', $this->debug);
 			ldap_set_option(null, LDAP_OPT_X_TLS_CERTFILE, getenv('TLS_CERT'));
 		}
 
 		if (getenv('TLS_CACERT') != '' && defined('LDAP_OPT_X_TLS_CACERTFILE')) {
-			cacti_log('NOTE: Settings TLS_CACERT to ' . getenv('TLS_CACERT'), false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('NOTE: Settings TLS_CACERT to ' . getenv('TLS_CACERT'), false, 'AUTH', $this->debug);
 			ldap_set_option(null, LDAP_OPT_X_TLS_CACERTFILE, getenv('TLS_CACERT'));
 		}
 
 		if (getenv('TLS_KEY') != '' && defined('LDAP_OPT_X_TLS_KEYFILE')) {
-			cacti_log('NOTE: Settings TLS_KEY to ' . getenv('TLS_KEY'), false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('NOTE: Settings TLS_KEY to ' . getenv('TLS_KEY'), false, 'AUTH', $this->debug);
 			ldap_set_option(null, LDAP_OPT_X_TLS_KEYFILE, getenv('TLS_KEY'));
 		}
 
 		if (getenv('TLS_CACERTDIR') != '' && defined('LDAP_OPT_X_TLS_CACERTDIR')) {
-			cacti_log('NOTE: Settings TLS_CACERTDIR to ' . getenv('TLS_CACERTDIR'), false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('NOTE: Settings TLS_CACERTDIR to ' . getenv('TLS_CACERTDIR'), false, 'AUTH', $this->debug);
 			ldap_set_option(null, LDAP_OPT_X_TLS_CACERTDIR, getenv('TLS_CACERTDIR'));
 		}
 
@@ -488,23 +520,23 @@ class Ldap {
 			// For good measure, we will use both the php function and set the environment
 			switch($cert) {
 				case LDAP_OPT_X_TLS_NEVER:
-					cacti_log('NOTE: Setting TLS Certificate Requirement to \'never\'', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+					cacti_log('NOTE: Setting TLS Certificate Requirement to \'never\'', false, 'AUTH', $this->debug);
 					putenv('TLS_REQCERT=never');
 					break;
 				case LDAP_OPT_X_TLS_HARD:
-					cacti_log('NOTE: Setting TLS Certificate Requirement to \'hard\'', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+					cacti_log('NOTE: Setting TLS Certificate Requirement to \'hard\'', false, 'AUTH', $this->debug);
 					putenv('TLS_REQCERT=hard');
 					break;
 				case LDAP_OPT_X_TLS_DEMAND:
-					cacti_log('NOTE: Setting TLS Certificate Requirement to \'demand\'', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+					cacti_log('NOTE: Setting TLS Certificate Requirement to \'demand\'', false, 'AUTH', $this->debug);
 					putenv('TLS_REQCERT=demand');
 					break;
 				case LDAP_OPT_X_TLS_ALLOW:
-					cacti_log('NOTE: Setting TLS Certificate Requirement to \'allow\'', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+					cacti_log('NOTE: Setting TLS Certificate Requirement to \'allow\'', false, 'AUTH', $this->debug);
 					putenv('TLS_REQCERT=allow');
 					break;
 				case LDAP_OPT_X_TLS_TRY:
-					cacti_log('NOTE: Setting TLS Certificate Requirement to \'try\'', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+					cacti_log('NOTE: Setting TLS Certificate Requirement to \'try\'', false, 'AUTH', $this->debug);
 					putenv('TLS_REQCERT=try');
 					break;
 			}
@@ -514,10 +546,10 @@ class Ldap {
 
 		// Walk through ldap servers for a valid connections
 		if ($this->encryption == '1') {
-			cacti_log('LDAP: Connect using ldaps://' . $this->host . ':' . $this->port_ssl, false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('LDAP: Connect using ldaps://' . $this->host . ':' . $this->port_ssl, false, 'AUTH', $this->debug);
 			$ldap_conn = ldap_connect('ldaps://' . $this->host . ':' . $this->port_ssl);
 		} else {
-			cacti_log('LDAP: Connect using ldap://'. $this->host . ':' . $this->port, false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			cacti_log('LDAP: Connect using ldap://'. $this->host . ':' . $this->port, false, 'AUTH', $this->debug);
 			$ldap_conn = ldap_connect($this->host, $this->port);
 		}
 
@@ -539,13 +571,13 @@ class Ldap {
 			/* set reasonable timeouts */
 			$network_timeout = read_config_option('ldap_network_timeout');
 			if (defined('LDAP_OPT_NETWORK_TIMEOUT')) {
-				cacti_log("NOTE: Setting Network Timeout to $network_timeout seconds", false, 'AUTH', POLLER_VERBOSITY_HIGH);
+				cacti_log("NOTE: Setting Network Timeout to $network_timeout seconds", false, 'AUTH', $this->debug);
 				ldap_set_option($ldap_conn, LDAP_OPT_NETWORK_TIMEOUT, $network_timeout);
 			}
 
 			$bind_timeout = read_config_option('ldap_bind_timeout');
 			if (defined('LDAP_OPT_TIMELIMIT')) {
-				cacti_log("NOTE: Setting Bind Timeout to $bind_timeout seconds", false, 'AUTH', POLLER_VERBOSITY_HIGH);
+				cacti_log("NOTE: Setting Bind Timeout to $bind_timeout seconds", false, 'AUTH', $this->debug);
 				ldap_set_option($ldap_conn, LDAP_OPT_TIMEOUT, $bind_timeout);
 			}
 
@@ -605,8 +637,12 @@ class Ldap {
 		$this->password = html_entity_decode($this->password, $this->GetMask(), 'UTF-8');
 		$this->dn = str_replace('<username>', $this->username, $this->dn);
 
+		if ($this->password == '') {
+			return LdapError::GetErrorDetails(LdapError::EmptyPassword);
+		}
+
 		/* Bind to the LDAP directory */
-		cacti_log('LDAP: Binding with "' . $this->dn . '"', false, 'AUTH', POLLER_VERBOSITY_HIGH);
+		cacti_log('LDAP: Binding with "' . $this->dn . '"', false, 'AUTH', $this->debug);
 		$ldap_response = ldap_bind($ldap_conn, $this->dn, $this->password);
 		if ($ldap_response) {
 			if ($this->group_require == 1) {
@@ -622,7 +658,7 @@ class Ldap {
 					/* Do a lookup to find this user's true DN. */
 					/* ldap_exop_whoami is not yet included in PHP. For reference, the
 					 * feature request: http://bugs.php.net/bug.php?id=42060
-					 * And the patch against lastest PHP release:
+					 * And the patch against latest PHP release:
 					 * http://cvsweb.netbsd.org/bsdweb.cgi/pkgsrc/databases/php-ldap/files/ldap-ctrl-exop.patch
 					*/
 					$true_dn_result = ldap_search($ldap_conn, $this->search_base, 'userPrincipalName=' . $this->dn, array('dn'));
@@ -667,7 +703,7 @@ class Ldap {
 				/* invalid credentials */
 				$output = LdapError::GetErrorDetails(LdapError::Failure, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x32) {
-				/* insuffient access */
+				/* insufficient access */
 				$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x51) {
 				/* unable to connect to server */
@@ -711,7 +747,7 @@ class Ldap {
 
 		if (cacti_sizeof($connection['output'])) {
 			$this->RestoreCactiHandler();
-			return $output;
+			return $connection['output'];
 		} elseif ($connection['ldap_conn'] === false) {
 			$this->RestoreCactiHandler();
 			return LdapError::GetErrorDetails(LdapError::MissingLdapObject, false, $this->host);
@@ -784,7 +820,7 @@ class Ldap {
 				/* invalid credentials */
 				$output = LdapError::GetErrorDetails(LdapError::Failure, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x32) {
-				/* insuffient access */
+				/* insufficient access */
 				$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x51) {
 				/* unable to connect to server */
@@ -819,7 +855,7 @@ class Ldap {
 
 		if (cacti_sizeof($connection['output'])) {
 			$this->RestoreCactiHandler();
-			return $output;
+			return $connection['output'];
 		} elseif ($connection['ldap_conn'] === false) {
 			$this->RestoreCactiHandler();
 			return LdapError::GetErrorDetails(LdapError::MissingLdapObject, false, $this->host);
@@ -896,7 +932,7 @@ class Ldap {
 				/* invalid credentials */
 				$output = LdapError::GetErrorDetails(LdapError::Failure, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x32) {
-				/* insuffient access */
+				/* insufficient access */
 				$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess, $ldap_conn, $this->host);
 			} elseif ($ldap_error == 0x51) {
 				/* unable to connect to server */
