@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -39,7 +39,7 @@ function exec_poll($command) {
 			$fp = popen($command, 'rb');
 		}
 
-		/* return if the popen command was not successfull */
+		/* return if the popen command was not successful */
 		if (!is_resource($fp)) {
 			cacti_log('WARNING; Problem with POPEN command.', false, 'POLLER');
 			return 'U';
@@ -102,7 +102,7 @@ function exec_poll_php($command, $using_proc_function, $pipes, $proc_fd) {
 				$fp = popen($command, 'rb');
 			}
 
-			/* return if the popen command was not successfull */
+			/* return if the popen command was not successful */
 			if (!is_resource($fp)) {
 				cacti_log('WARNING; Problem with POPEN command.', false, 'POLLER');
 				return 'U';
@@ -159,6 +159,104 @@ function exec_background($filename, $args = '', $redirect_args = '') {
 	}
 }
 
+/**
+ * exec_with_timeout - Execute a command and return it's output. Either wait until the
+ * command exits or the timeout has expired.
+ *
+ * @param  (string)      $cmd          Command to execute.
+ * @param  (string)      $output       A return array of output.
+ * @param  (int)         $return_code  The return code from the script
+ * @param  (int)         $timeout      Timeout in seconds.
+ *
+ * @return (string|bool) Either the last line of output or false on error
+ */
+function exec_with_timeout($cmd, &$output, &$return_code, $timeout = 5) {
+	// File descriptors passed to the process.
+	$descriptors = array(
+		0 => array('pipe', 'r'),  // stdin
+		1 => array('pipe', 'w'),  // stdout
+		2 => array('pipe', 'w')   // stderr
+	);
+
+	// Start the process.
+	$process = proc_open('exec ' . $cmd, $descriptors, $pipes);
+
+	if (!is_resource($process)) {
+		return false;
+	}
+
+	// Set the stdout stream to non-blocking.
+	stream_set_blocking($pipes[1], 0);
+
+	// Set the stderr stream to non-blocking.
+	stream_set_blocking($pipes[2], 0);
+
+	// Turn the timeout into microseconds.
+	$timeout = (int) $timeout * 1000000;
+
+	// Output buffer.
+	$buffer = '';
+
+	// While we have time to wait.
+	while ($timeout > 0) {
+		$start = microtime(true);
+
+		// Wait until we have output or the timer expired.
+		$read  = array($pipes[1]);
+		$write = array();
+		$other = array();
+		stream_select($read, $write, $other, 0, $timeout);
+
+		// Get the status of the process.
+		// Do this before we read from the stream,
+		// this way we can't lose the last bit of output if the process dies between these functions.
+		$status = proc_get_status($process);
+
+		// Read the contents from the buffer.
+		// This function will always return immediately as the stream is non-blocking.
+		$buffer .= stream_get_contents($pipes[1]);
+
+		if (!$status['running']) {
+			// Break from this loop if the process exited before the timeout.
+			break;
+		}
+
+		// Subtract the number of microseconds that we waited.
+		$timeout -= (int) (microtime(true) - $start) * 1000000;
+	}
+
+	// Check if there were any errors.
+	$errors = stream_get_contents($pipes[2]);
+
+	if (isset($status['exitcode'])) {
+		$return_code = $status['exitcode'];
+	} else {
+		$return_code = 1;
+	}
+
+	if (!empty($errors)) {
+		return false;
+	}
+
+	// Kill the process in case the timeout expired and it's still running.
+	// If the process already exited this won't do anything.
+	proc_terminate($process, 9);
+
+	// Close all streams.
+	fclose($pipes[0]);
+	fclose($pipes[1]);
+	fclose($pipes[2]);
+
+	proc_close($process);
+
+	if ($buffer != '') {
+		$output = explode("\n", $buffer);
+		return(end($output));
+	} else {
+		return;
+	}
+}
+
 function file_escaped($file) {
 	if (substr($file, 0, 1) == '"' && substr($file, -1, 1) == '"') {
 		return true;
@@ -375,7 +473,7 @@ function poller_update_poller_reindex_from_buffer($host_id, $data_query_id, &$re
 	/* use a reasonable insert buffer, the default is 1MByte */
 	$max_packet   = 256000;
 
-	/* setup somme defaults */
+	/* setup some defaults */
 	$overhead     = strlen($sql_prefix) + strlen($sql_suffix);
 	$buf_len      = 0;
 	$buf_count    = 0;
@@ -451,10 +549,10 @@ function process_poller_output(&$rrdtool_pipe, $remainder = 0) {
 		pi.rrd_path, pi.rrd_name, pi.rrd_num
 		FROM poller_output AS po
 		INNER JOIN poller_item AS pi
-		ON po.local_data_id=pi.local_data_id
-		AND po.rrd_name=pi.rrd_name
+		ON po.local_data_id = pi.local_data_id
+		AND po.rrd_name = pi.rrd_name
 		INNER JOIN data_local AS dl
-		ON dl.id=po.local_data_id
+		ON dl.id = po.local_data_id
 		ORDER BY po.local_data_id
 		$limit");
 
@@ -559,6 +657,11 @@ function process_poller_output(&$rrdtool_pipe, $remainder = 0) {
 			$rrd_name  = $item['rrd_name'];
 
 			if (isset($rrd_update_array[$rrd_path]['times'][$unix_time])) {
+				/**
+				 * Check to see if we have partial data sources.  If so
+				 * we did not get a full update, so we should not be removing
+				 * those data sources from the $rrd_update_array yet.
+				 */
 				if ($item['rrd_num'] <= cacti_sizeof($rrd_update_array[$rrd_path]['times'][$unix_time])) {
 					$data_ids[] = $item['local_data_id'];
 					$k++;
@@ -596,17 +699,17 @@ function process_poller_output(&$rrdtool_pipe, $remainder = 0) {
 
 		/* to much records in poller_output, process in chunks */
 		if ($rows && $remainder == $max_rows) {
-			$rrds_processed += process_poller_output($rrdtool_pipe, $rows < $max_rows ? $rows : $max_rows);
-
 			$running = db_fetch_cell('SELECT COUNT(*)
 				FROM poller_time
 				WHERE end_time = "0000-00-00"');
 
+			$rrds_processed += process_poller_output($rrdtool_pipe, $rows < $max_rows ? $rows : $max_rows);
+
 			if ($running == 0 && !$checked_bad) {
 				// Remove recently deleted items from the poller_output table
 				db_execute('DELETE FROM poller_output WHERE local_data_id NOT IN (SELECT id FROM data_local)');
 
-				// Itentify data sources that are somehow not aligned
+				// Identify data sources that are somehow not aligned
 				$items = db_fetch_assoc('SELECT rrd_num,
 					COUNT(DISTINCT po.local_data_id, po.rrd_name) AS ids, dt.name, dl.host_id,
 					GROUP_CONCAT(DISTINCT po.local_data_id) AS local_data_ids
@@ -1171,7 +1274,7 @@ function md5sum_path($path, $recursive = true) {
 }
 
 /**
- * poller_puth_to_remote_db_connect - given the device or poller_id connect
+ * poller_push_to_remote_db_connect - given the device or poller_id connect
  *   to the data collector.
  *
  * @param  (int)    device_or_poller - the id of the object
@@ -1399,6 +1502,9 @@ function replicate_out($remote_poller_id = 1, $class = 'all') {
 		$data = db_fetch_assoc('SELECT * FROM user_auth_realm');
 		replicate_out_table($rcnn_id, $data, 'user_auth_realm', $remote_poller_id);
 
+		$data = db_fetch_assoc('SELECT * FROM user_auth_row_cache');
+		replicate_out_table($rcnn_id, $data, 'user_auth_row_cache', $remote_poller_id);
+
 		$data = db_fetch_assoc('SELECT * FROM user_domains');
 		replicate_out_table($rcnn_id, $data, 'user_domains', $remote_poller_id);
 
@@ -1742,70 +1848,87 @@ function poller_push_reindex_data_to_poller($device_id = 0, $data_query_id = 0,
 		$db_cnn_id = $remote_db_cnn_id;
 	}
 
+	$sql_params  = array();
+	$sql_params1 = array();
 	$sql_where   = '';
 	$sql_where1  = '';
-	$sql_where  .= $device_id > 0 ? 'WHERE host_id = ' . $device_id:'';
-	$sql_where1 .= $device_id > 0 ? ' AND host_id = ' . $device_id:'';
+	$sql_where2  = '';
 
-	if ($data_query_id > 0) {
-		$sql_where .= ($sql_where != '' ? ' AND':'WHERE ') . ' snmp_query_id = ' . $data_query_id;
-		$sql_where1 .= ' AND snmp_query_id = ' . $data_query_id;
+	if ($device_id > 0) {
+		$sql_where  .= 'WHERE host_id = ?';
+		$sql_where1 .= 'WHERE host_id = ?';
+
+		$sql_params[]  = $device_id;
+		$sql_params1[] = $device_id;
 	}
 
-	// Give the snmp query upto an hour to run
-	$min_reindex_cache = db_fetch_cell("SELECT MIN(UNIX_TIMESTAMP(last_updated)-3600)
+	if ($data_query_id > 0) {
+		$sql_where  .= ($sql_where  != '' ? ' AND':'WHERE') . ' snmp_query_id = ?';
+		$sql_where1 .= ($sql_where1 != '' ? ' AND':'WHERE') . ' snmp_query_id = ?';
+
+		$sql_params[]  = $data_query_id;
+		$sql_params1[] = $data_query_id;
+	}
+
+	// Give the snmp query up to an hour to run
+	$min_reindex_cache = db_fetch_cell_prepared("SELECT MIN(UNIX_TIMESTAMP(last_updated)-3600)
 		FROM host_snmp_cache
-		$sql_where");
+		$sql_where",
+		$sql_params);
 
 	if (!$force) {
+		$sql_where2    = $sql_where1 . ($sql_where1 != '' ? ' AND':'WHERE') . ' UNIX_TIMESTAMP(last_updated) > ?';
+		$sql_params2   = $sql_params1;
+		$sql_params2[] = $min_reindex_cache;
+
 		$recache_hosts = array_rekey(
 			db_fetch_assoc_prepared("SELECT DISTINCT host_id
 				FROM host_snmp_cache
-				WHERE UNIX_TIMESTAMP(last_updated) > ?
-				$sql_where1",
-				array($min_reindex_cache)),
+				$sql_where2",
+				$sql_params2),
 			'host_id', 'host_id'
 		);
 	} else {
 		$recache_hosts = array_rekey(
 			db_fetch_assoc_prepared("SELECT DISTINCT host_id
 				FROM host_snmp_cache
-				WHERE UNIX_TIMESTAMP(last_updated) > 0
 				$sql_where1",
-				array($min_reindex_cache)),
+				$sql_params1),
 			'host_id', 'host_id'
 		);
 	}
 
 	if (cacti_sizeof($recache_hosts)) {
-		$local_data_ids = db_fetch_assoc("SELECT *
+		$sql_where1 .= ($sql_where1 != '' ? ' AND':'WHERE') . ' host_id IN (' . implode(', ', $recache_hosts) . ')';
+
+		$local_data_ids = db_fetch_assoc_prepared("SELECT *
 			FROM data_local
-			WHERE host_id IN (" . implode(', ', $recache_hosts) . ")
-			$sql_where1");
+			$sql_where1",
+			$sql_params1);
 
 		replicate_table_to_poller($db_cnn_id, $local_data_ids, 'data_local');
 
-		$local_graph_ids = db_fetch_assoc("SELECT *
+		$local_graph_ids = db_fetch_assoc_prepared("SELECT *
 			FROM graph_local
-			WHERE host_id IN (" . implode(', ', $recache_hosts) . ")
-			$sql_where1");
+			$sql_where1",
+			$sql_params1);
 
 		replicate_table_to_poller($db_cnn_id, $local_graph_ids, 'graph_local');
 
-		$host_snmp_cache = db_fetch_assoc("SELECT *
+		$host_snmp_cache = db_fetch_assoc_prepared("SELECT *
 			FROM host_snmp_cache
-			WHERE host_id IN (" . implode(', ', $recache_hosts) . ")
-			$sql_where1");
+			$sql_where1",
+			$sql_params1);
 
 		replicate_table_to_poller($db_cnn_id, $host_snmp_cache, 'host_snmp_cache');
 
 		// TODO: Make schema's equivalent renamed snmp_query_id to data_query_id everywhere
 		$sql_where1 = str_replace('snmp_query_id', 'data_query_id', $sql_where1);
 
-		$poller_reindex = db_fetch_assoc("SELECT *
+		$poller_reindex = db_fetch_assoc_prepared("SELECT *
 			FROM poller_reindex
-			WHERE host_id IN (" . implode(', ', $recache_hosts) . ")
-			$sql_where1");
+			$sql_where1",
+			$sql_params1);
 
 		replicate_table_to_poller($db_cnn_id, $poller_reindex, 'poller_reindex', array('assert_value'));
 	}
