@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -39,12 +39,12 @@ function repopulate_poller_cache() {
 
 	include_once($config['library_path'] . '/api_data_source.php');
 
-	$poller_data    = db_fetch_assoc('SELECT ' . SQL_NO_CACHE . ' dl.*, h.poller_id
+	$poller_data = db_fetch_assoc('SELECT ' . SQL_NO_CACHE . ' dl.*, h.poller_id
 		FROM data_local AS dl
 		INNER JOIN host AS h
 		ON dl.host_id=h.id
 		WHERE dl.snmp_query_id = 0 OR (dl.snmp_query_id > 0 AND dl.snmp_index != "")
-		ORDER BY h.poller_id ASC');
+		ORDER BY h.poller_id ASC, h.id ASC');
 
 	$poller_items   = array();
 	$local_data_ids = array();
@@ -101,6 +101,12 @@ function repopulate_poller_cache() {
 		ON dtr.data_input_field_id = dif.id
 		WHERE dtr.local_data_id = 0
 		GROUP BY dtr.data_template_id, dif.data_name');
+
+	if (isset($_SESSION['sess_user_id'])) {
+		cacti_log('NOTE: Poller Cache repopulated by user ' . get_username($_SESSION['sess_user_id']), false, 'WEBUI');
+	} else {
+		cacti_log('NOTE: Poller Cache repopulated by cli script');
+	}
 }
 
 function update_poller_cache_from_query($host_id, $data_query_id, $local_data_ids) {
@@ -197,18 +203,19 @@ function update_poller_cache($data_source, $commit = false) {
 			$field = data_query_field_list($data_input['data_template_data_id']);
 
 			$params = array();
-			if ($field['output_type'] != '') {
+			if (cacti_sizeof($field) && $field['output_type'] != '') {
 				$output_type_sql = ' AND sqgr.snmp_query_graph_id = ' . $field['output_type'];
 			} else {
 				$output_type_sql = '';
 			}
+
 			$params[] = $data_input['data_template_id'];
 			$params[] = $data_source['id'];
 
 			$outputs = db_fetch_assoc_prepared('SELECT DISTINCT ' . SQL_NO_CACHE . "
-				sqgr.snmp_field_name, dtr.id as data_template_rrd_id
+				sqgr.snmp_field_name, dtr.id AS data_template_rrd_id
 				FROM snmp_query_graph_rrd AS sqgr
-				INNER JOIN data_template_rrd AS dtr FORCE INDEX (local_data_id)
+				INNER JOIN data_template_rrd AS dtr
 				ON sqgr.data_template_rrd_id = dtr.local_data_template_rrd_id
 				WHERE sqgr.data_template_id = ?
 				AND dtr.local_data_id = ?
@@ -565,7 +572,7 @@ function poller_update_poller_cache_from_buffer($local_data_ids, &$poller_items,
 	/* use a reasonable insert buffer, the default is 1MByte */
 	$max_packet   = 256000;
 
-	/* setup somme defaults */
+	/* setup some defaults */
 	$overhead     = strlen($sql_prefix) + strlen($sql_suffix);
 	$buf_len      = 0;
 	$buf_count    = 0;
@@ -656,6 +663,12 @@ function poller_update_poller_cache_from_buffer($local_data_ids, &$poller_items,
 			}
 		}
 	}
+
+	/**
+	 * Save the last time a device/site was created/updated
+	 * for Caching.
+	 */
+	set_config_option('time_last_change_poller_item', time());
 }
 
 /** for a given data template, update all input data and the poller cache
@@ -876,21 +889,15 @@ function data_input_whitelist_check($data_input_id) {
 	}
 }
 
-function utilities_get_mysql_recommendations() {
-	global $config, $local_db_cnn_id;
+function utilities_get_mysql_info($poller_id = 1) {
+	global $local_db_cnn_id;
 
-	// MySQL/MariaDB Important Variables
-	// Assume we are successfully, until we aren't!
-	$result = DB_STATUS_SUCCESS;
-
-	if ($config['poller_id'] == 1) {
+	if ($poller_id == 1) {
 		$variables = array_rekey(db_fetch_assoc('SHOW GLOBAL VARIABLES'), 'Variable_name', 'Value');
 	} else {
 		$variables = array_rekey(db_fetch_assoc('SHOW GLOBAL VARIABLES', false, $local_db_cnn_id), 'Variable_name', 'Value');
 	}
 
-	$memInfo = utilities_get_system_memory();
-
 	if (strpos($variables['version'], 'MariaDB') !== false) {
 		$database = 'MariaDB';
 		$version  = str_replace('-MariaDB', '', $variables['version']);
@@ -898,7 +905,7 @@ function utilities_get_mysql_recommendations() {
 		if (isset($variables['innodb_version'])) {
 			$link_ver = substr($variables['innodb_version'], 0, 3);
 		} else {
-			$link_ver = '5.5';
+			$link_ver = $version;
 		}
 	} else {
 		$database = 'MySQL';
@@ -906,6 +913,30 @@ function utilities_get_mysql_recommendations() {
 		$link_ver = substr($variables['version'], 0, 3);
 	}
 
+	return array(
+		'database'  => $database,
+		'version'   => $version,
+		'link_ver'  => $link_ver,
+		'variables' => $variables
+	);
+}
+
+function utilities_get_mysql_recommendations() {
+	global $config, $local_db_cnn_id;
+
+	// MySQL/MariaDB Important Variables
+	// Assume we are successfully, until we aren't!
+	$result = DB_STATUS_SUCCESS;
+
+	$memInfo = utilities_get_system_memory();
+
+	$mysql_info = utilities_get_mysql_info($config['poller_id']);
+
+	$database  = $mysql_info['database'];
+	$version   = $mysql_info['version'];
+	$link_ver  = $mysql_info['link_ver'];
+	$variables = $mysql_info['variables'];
+
 	$recommendations = array(
 		'version' => array(
 			'value' => '5.6',
@@ -1017,10 +1048,16 @@ function utilities_get_mysql_recommendations() {
 			'comment' => __('When executing subqueries, having a larger temporary table size, keep those temporary tables in memory.')
 			),
 		'join_buffer_size' => array(
-			'value'   => '3.2',
-			'measure' => 'pmem',
+			'value'   => '80',
+			'measure' => 'cmem',
 			'class'   => 'warning',
-			'comment' => __('When performing joins, if they are below this size, they will be kept in memory and never written to a temporary file.')
+			'comment' => __('When performing joins, if they are below this size, they will be kept in memory and never written to a temporary file.  As this is a per connection memory allocation, care must be taken not to increase it too high.  The sum of the join_buffer_size + sort_buffer_size + read_buffer_size + read_rnd_buffer_size + thread_stack + binlog_cache_size + Core MySQL/MariaDB memory should be below 80%.  If the recommendation is negative, you must decrease this and or the sort_buffer_size until the recommendation fits within the allowable memory.')
+			),
+		'sort_buffer_size' => array(
+			'value'   => '80',
+			'measure' => 'cmem',
+			'class'   => 'warning',
+			'comment' => __('When performing joins, if they are below this size, they will be kept in memory and never written to a temporary file.  As this is a per connection memory allocation, care must be taken not to increase it too high.  The sum of the join_buffer_size + sort_buffer_size + read_buffer_size + read_rnd_buffer_size + thread_stack + binlog_cache_size + Core MySQL/MariaDB memory should be below 80%.  If the recommendation is negative, you must decrease this and or the sort_buffer_size until the recommendation fits within the allowable memory.')
 			),
 		'innodb_file_per_table' => array(
 			'value'   => 'ON',
@@ -1083,7 +1120,7 @@ function utilities_get_mysql_recommendations() {
 					'comment' => __('With modern SSD type storage, having multiple io threads is advantageous for applications with high io characteristics.')
 					)
 			);
-		} elseif (version_compare($variables['innodb_version'], '10.5', '<')) {
+		} elseif ($database == 'MariaDB' && version_compare($variables['innodb_version'], '10.5', '<') || $database == 'MySQL') {
 			$recommendations += array(
 				'innodb_flush_log_at_timeout' => array(
 					'value'   => '3',
@@ -1104,7 +1141,7 @@ function utilities_get_mysql_recommendations() {
 					'value' => '16',
 					'measure' => 'pinst',
 					'class' => 'warning',
-					'comment' => __('%s will divide the innodb_buffer_pool into memory regions to improve performance for versions of MariaDB less than 10.5.  The max value is 64.  When your innodb_buffer_pool is less than 1GB, you should use the pool size divided by 128MB.  Continue to use this equation upto the max of 64.', $database)
+					'comment' => ($database == 'MySQL' ? __('%s will divide the innodb_buffer_pool into memory regions to improve performance for versions of MySQL upto and including MySQL 8.0.  The max value is 64, but should not exceed more than the number of CPU cores/threads.  When your innodb_buffer_pool is less than 1GB, you should use the pool size divided by 128MB.  Continue to use this equation up to the max of the number of CPU cores or 64.', $database): __('%s will divide the innodb_buffer_pool into memory regions to improve performance for versions of MariaDB less than 10.5.  The max value is 64, but should not exceed more than the number of CPU cores/threads.  When your innodb_buffer_pool is less than 1GB, you should use the pool size divided by 128MB.  Continue to use this equation up to the max the number of CPU cores or 64.', $database))
 					),
 				'innodb_io_capacity' => array(
 					'value' => '5000',
@@ -1269,6 +1306,137 @@ function utilities_get_mysql_recommendations() {
 				$passed = ($variables[$name] >= ($r['value']*$totalMem/100));
 				$value_display = round($variables[$name]/1024/1024, 2) . ' M';
 				$value_recommend = round($r['value']*$totalMem/100/1024/1024, 2) . ' M';
+				break;
+			case 'cmem':
+				if (isset($memInfo['MemTotal'])) {
+					$totalMem = $memInfo['MemTotal'];
+				} elseif (isset($memInfo['TotalVisibleMemorySize'])) {
+					$totalMem = $memInfo['TotalVisibleMemorySize'];
+				} else {
+					break;
+				}
+
+				if ($config['poller_id'] == 1) {
+					$maxConnections = db_fetch_cell('SELECT @@GLOBAL.max_connections');
+				} else {
+					$maxConnections = db_fetch_cell('SELECT @@GLOBAL.max_connections', '', false, $local_db_cnn_id);
+				}
+
+				if ($name == 'sort_buffer_size') {
+					if ($config['poller_id'] == 1) {
+						if (($database == 'MySQL' && version_compare($version, '8.0', '<')) || $database == 'MariaDB') {
+							$totalMemorySans = db_fetch_cell('SELECT @@GLOBAL.key_buffer_size +
+								@@GLOBAL.query_cache_size +
+								@@GLOBAL.tmp_table_size +
+								@@GLOBAL.innodb_buffer_pool_size +
+								@@GLOBAL.innodb_log_buffer_size
+								+ @@GLOBAL.max_connections * (
+									@@GLOBAL.join_buffer_size +
+									@@GLOBAL.read_buffer_size +
+									@@GLOBAL.read_rnd_buffer_size +
+									@@GLOBAL.thread_stack +
+									@@GLOBAL.binlog_cache_size)');
+						} else {
+							$totalMemorySans = db_fetch_cell('SELECT @@GLOBAL.key_buffer_size +
+								@@GLOBAL.tmp_table_size +
+								@@GLOBAL.innodb_buffer_pool_size +
+								@@GLOBAL.innodb_log_buffer_size
+								+ @@GLOBAL.max_connections * (
+									@@GLOBAL.join_buffer_size +
+									@@GLOBAL.read_buffer_size +
+									@@GLOBAL.read_rnd_buffer_size +
+									@@GLOBAL.thread_stack +
+									@@GLOBAL.binlog_cache_size)');
+						}
+					} else {
+						if (($database == 'MySQL' && version_compare($version, '8.0', '<')) || $database == 'MariaDB') {
+							$totalMemorySans = db_fetch_cell('SELECT @@GLOBAL.key_buffer_size +
+								@@GLOBAL.query_cache_size +
+								@@GLOBAL.tmp_table_size +
+								@@GLOBAL.innodb_buffer_pool_size +
+								@@GLOBAL.innodb_log_buffer_size
+								+ @@GLOBAL.max_connections * (
+									@@GLOBAL.join_buffer_size +
+									@@GLOBAL.read_buffer_size +
+									@@GLOBAL.read_rnd_buffer_size +
+									@@GLOBAL.thread_stack +
+									@@GLOBAL.binlog_cache_size)', '', false, $local_db_cnn_id);
+						} else {
+							$totalMemorySans = db_fetch_cell('SELECT @@GLOBAL.key_buffer_size +
+								@@GLOBAL.tmp_table_size +
+								@@GLOBAL.innodb_buffer_pool_size +
+								@@GLOBAL.innodb_log_buffer_size
+								+ @@GLOBAL.max_connections * (
+									@@GLOBAL.join_buffer_size +
+									@@GLOBAL.read_buffer_size +
+									@@GLOBAL.read_rnd_buffer_size +
+									@@GLOBAL.thread_stack +
+									@@GLOBAL.binlog_cache_size)', '', false, $local_db_cnn_id);
+						}
+					}
+				} else {
+					if ($config['poller_id'] == 1) {
+						if (($database == 'MySQL' && version_compare($version, '8.0', '<')) || $database == 'MariaDB') {
+							$totalMemorySans = db_fetch_cell('SELECT @@GLOBAL.key_buffer_size +
+								@@GLOBAL.query_cache_size +
+								@@GLOBAL.tmp_table_size +
+								@@GLOBAL.innodb_buffer_pool_size +
+								@@GLOBAL.innodb_log_buffer_size
+								+ @@GLOBAL.max_connections * (
+									@@GLOBAL.sort_buffer_size +
+									@@GLOBAL.read_buffer_size +
+									@@GLOBAL.read_rnd_buffer_size +
+									@@GLOBAL.thread_stack +
+									@@GLOBAL.binlog_cache_size)');
+						} else {
+							$totalMemorySans = db_fetch_cell('SELECT @@GLOBAL.key_buffer_size +
+								@@GLOBAL.tmp_table_size +
+								@@GLOBAL.innodb_buffer_pool_size +
+								@@GLOBAL.innodb_log_buffer_size
+								+ @@GLOBAL.max_connections * (
+									@@GLOBAL.sort_buffer_size +
+									@@GLOBAL.read_buffer_size +
+									@@GLOBAL.read_rnd_buffer_size +
+									@@GLOBAL.thread_stack +
+									@@GLOBAL.binlog_cache_size)');
+						}
+					} else {
+						if (($database == 'MySQL' && version_compare($version, '8.0', '<')) || $database == 'MariaDB') {
+							$totalMemorySans = db_fetch_cell('SELECT @@GLOBAL.key_buffer_size +
+								@@GLOBAL.query_cache_size +
+								@@GLOBAL.tmp_table_size +
+								@@GLOBAL.innodb_buffer_pool_size +
+								@@GLOBAL.innodb_log_buffer_size
+								+ @@GLOBAL.max_connections * (
+									@@GLOBAL.sort_buffer_size +
+									@@GLOBAL.read_buffer_size +
+									@@GLOBAL.read_rnd_buffer_size +
+									@@GLOBAL.thread_stack +
+									@@GLOBAL.binlog_cache_size)', '', false, $local_db_cnn_id);
+						} else {
+							$totalMemorySans = db_fetch_cell('SELECT @@GLOBAL.key_buffer_size +
+								@@GLOBAL.tmp_table_size +
+								@@GLOBAL.innodb_buffer_pool_size +
+								@@GLOBAL.innodb_log_buffer_size
+								+ @@GLOBAL.max_connections * (
+									@@GLOBAL.sort_buffer_size +
+									@@GLOBAL.read_buffer_size +
+									@@GLOBAL.read_rnd_buffer_size +
+									@@GLOBAL.thread_stack +
+									@@GLOBAL.binlog_cache_size)', '', false, $local_db_cnn_id);
+						}
+					}
+				}
+
+				$remainingMem = ($totalMem * 0.8) - $totalMemorySans;
+
+				$recommendation = $remainingMem / $maxConnections;
+
+				$compare = '<=';
+				$passed = ($variables[$name] >= ($recommendation/1024/1024)) && $recommendation > 0;
+				$value_display = round($variables[$name]/1024/1024, 2) . ' M';
+				$value_recommend = round($recommendation/1024/1024, 2) . ' M';
+
 				break;
 			case 'pinst':
 				$compare = '>=';
@@ -1276,6 +1444,16 @@ function utilities_get_mysql_recommendations() {
 				// Divide the buffer pool size by 128MB, and ensure 1 or more
 				$pool_instances = round(($innodb_pool_size / 1024 / 1024 / 128) + 0.5);
 
+				if ($config['cacti_server_os'] == 'win32') {
+					$nproc = getenv('NUMBER_OF_PROCESSORS');
+				} else {
+					$nproc = system('nproc');
+				}
+
+				if ($pool_instances > $nproc) {
+					$pool_instances = $nproc;
+				}
+
 				if ($pool_instances < 1) {
 					$pool_instances = 1;
 				} elseif ($pool_instances > 64) {
@@ -1486,6 +1664,7 @@ function utility_php_verify_extensions(&$extensions, $source) {
 			'gd'        => array('cli' => false, 'web' => false),
 			'gmp'       => array('cli' => false, 'web' => false),
 			'hash'      => array('cli' => false, 'web' => false),
+			'intl'      => array('cli' => false, 'web' => false),
 			'json'      => array('cli' => false, 'web' => false),
 			'ldap'      => array('cli' => false, 'web' => false),
 			'mbstring'  => array('cli' => false, 'web' => false),
@@ -1504,12 +1683,13 @@ function utility_php_verify_extensions(&$extensions, $source) {
 
 		if ($config['cacti_server_os'] == 'unix') {
 			$extensions['posix'] = array('cli' => false, 'web' => false);
+			$extensions['pcntl'] = array('cli' => false, 'web' => true);
 		} else {
 			$extensions['com_dotnet'] = array('cli' => false, 'web' => false);
 		}
 	}
 
-	uksort($extensions, "utility_php_sort_extensions");
+	uksort($extensions, 'utility_php_sort_extensions');
 
 	foreach ($extensions as $name=>$extension) {
 		if (extension_loaded($name)){
@@ -1580,10 +1760,17 @@ function utility_php_verify_recommends(&$recommends, $source) {
 	$memory_limit   = utility_get_formatted_bytes($memory_ini, 'M', $memory_ini, 'B');
 
 	$execute_time   = ini_get('max_execution_time');
+	$ini_values     = parse_ini_file(get_cfg_var('cfg_file_path'));
 
 	$timezone       = ini_get('date.timezone');
 
 	$recommends = array(
+		array(
+			'name'        => 'location',
+			'value'       => get_cfg_var('cfg_file_path'),
+			'current'     => get_cfg_var('cfg_file_path'),
+			'status'      => 2,
+		),
 		array(
 			'name'        => 'version',
 			'value'       => $rec_version,
@@ -1594,19 +1781,19 @@ function utility_php_verify_recommends(&$recommends, $source) {
 			'name'        => 'memory_limit',
 			'value'       => $rec_memory_mb,
 			'current'     => $memory_ini,
-			'status'      => ($memory_limit <= 0 || $memory_limit >= $rec_memory) ? DB_STATUS_SUCCESS : DB_STATUS_WARNING,
+			'status'      => (($memory_limit <= 0 || $memory_limit >= $rec_memory) ? DB_STATUS_SUCCESS :($memory_limit != $ini_values['memory_limit'] ? DB_STATUS_RESTART :  DB_STATUS_WARNING)),
 		),
 		array(
 			'name'        => 'max_execution_time',
 			'value'       => $rec_execute,
 			'current'     => $execute_time,
-			'status'      => ($execute_time <= 0 || $execute_time >= $rec_execute) ? DB_STATUS_SUCCESS : DB_STATUS_WARNING,
+			'status'      => (($execute_time <= 0 || $execute_time >= $rec_execute) ? DB_STATUS_SUCCESS : ($execute_time != $ini_values['max_execution_time'] ? DB_STATUS_RESTART : DB_STATUS_WARNING)),
 		),
 		array(
 			'name'        => 'date.timezone',
 			'value'       => '<timezone>',
 			'current'     => $timezone,
-			'status'      => ($timezone ? DB_STATUS_SUCCESS : DB_STATUS_ERROR),
+			'status'      => ($timezone ? DB_STATUS_SUCCESS : ($timezone != $ini_values['date.timezone'] ? DB_STATUS_RESTART : DB_STATUS_ERROR)),
 		),
 	);
 }
@@ -1617,7 +1804,7 @@ function utility_php_set_recommends_text(&$recs) {
 			if (cacti_sizeof($recommends)) {
 				foreach ($recommends as $index => $recommend) {
 					if ($recommend['name'] == 'version') {
-						$recs[$name][$index]['description'] = __('PHP %s is the mimimum version', $recommend['value']);
+						$recs[$name][$index]['description'] = __('PHP %s is the minimum version', $recommend['value']);
 					} elseif ($recommend['name'] == 'memory_limit') {
 						$recs[$name][$index]['description'] = __('A minimum of %s memory limit', $recommend['value']);
 					} elseif ($recommend['name'] == 'max_execution_time') {
