@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -48,12 +48,11 @@ require_once($config['base_path'] . '/lib/utility.php');
 if ($config['poller_id'] > 1) {
 	if ($config['connection'] == 'online') {
 		db_force_remote_cnn();
-	} else {
+	} elseif (debounce_run_notification('db_offline')) {
 		cacti_log('WARNING: Main Cacti database offline or in recovery.  Can not run automation', false, 'AUTOM8');
 		admin_email(__('Cacti System Warning'), __('WARNING: Main Cacti database offline or in recovery'));
+		exit(1);
 	}
-
-	exit(1);
 }
 
 /** sig_handler - provides a generic means to catch exceptions to the Cacti log.
