@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -35,6 +35,7 @@ require(__DIR__ . '/include/cli_check.php');
 require_once($config['base_path'] . '/lib/poller.php');
 require_once($config['base_path'] . '/lib/boost.php');
 require_once($config['base_path'] . '/lib/dsstats.php');
+require_once($config['base_path'] . '/lib/rrdcheck.php');
 require_once($config['base_path'] . '/lib/rrd.php');
 
 /* get the boost polling cycle */
@@ -52,7 +53,7 @@ $child    = false;
 /* for releasing lock on SIGNAL */
 $current_lock = false;
 
-global $child, $next_run_time, $current_lock;
+global $child, $next_run_time, $archive_table, $current_lock;
 
 if (cacti_sizeof($parms)) {
 	foreach($parms as $parameter) {
@@ -153,11 +154,14 @@ if ($child == false) {
 
 		/* Launch the boost children */
 		if ($continue) {
+			/* Allow mysql to flush the rename transaction */
+			sleep(7);
+
 			boost_launch_children();
 
 			/* Wait for all processes to continue */
-			while (boost_processes_running()) {
-				boost_debug('Sleeping for 2 seconds.');
+			while ($running = boost_processes_running()) {
+				boost_debug(sprintf('%s Processes Running, Sleeping for 2 seconds.', $running));
 				sleep(2);
 			}
 
@@ -196,6 +200,7 @@ if ($child == false) {
 
 			if ($rrd_updates > 0) {
 				dsstats_boost_bottom();
+				rrdcheck_boost_bottom();
 
 				api_plugin_hook('boost_poller_bottom');
 			}
@@ -255,8 +260,10 @@ function sig_handler($signo) {
 			/* ignore all other signals */
 	}
 
-	if ($current_lock !== false) {
+	if ($current_lock !== false && $child) {
 		db_execute("SELECT RELEASE_LOCK('boost.single_ds.$current_lock')");
+	} elseif (!$child) {
+		db_execute("SELECT RELEASE_ALL_LOCKS()");
 	}
 }
 
@@ -278,20 +285,16 @@ function boost_kill_running_processes() {
 }
 
 function boost_processes_running() {
-	$finished = db_fetch_cell('SELECT COUNT(*)
+	$running = db_fetch_cell('SELECT COUNT(*)
 		FROM processes
 		WHERE tasktype = "boost"
 		AND taskname = "child"');
 
-	if ($finished == 0) {
-		return false;
-	}
-
-	return true;
+	return $running;
 }
 
 function boost_prepare_process_table() {
-	global $start_time, $max_run_duration, $config, $database_default, $debug, $get_memory, $memory_used;
+	global $start_time, $archive_table, $max_run_duration, $config, $database_default, $debug, $get_memory, $memory_used;
 
 	boost_debug('Parallel Process Setup Begins.');
 
@@ -307,8 +310,7 @@ function boost_prepare_process_table() {
 			/* if the runtime was exceeded, allow the next process to run */
 			if ($previous_start_time + $max_run_duration < $start_time) {
 				cacti_log('WARNING: Detected Poller Boost Overrun, Possible Boost Poller Crash', false, 'BOOST SVR');
-				admin_email(__('Cacti System Warning'), __('WARNING: Detected Poller Boost Overrun, Possible Boost Poller Crash', 'BOOST SVR'  ));
-
+				admin_email(__('Cacti System Warning'), __('WARNING: Detected Poller Boost Overrun, Possible Boost Poller Crash', 'BOOST SVR'));
 			}
 		}
 	}
@@ -319,7 +321,7 @@ function boost_prepare_process_table() {
 
 	$delayed_inserts = db_fetch_row("SHOW STATUS LIKE 'Not_flushed_delayed_rows'");
 
-	while ($delayed_inserts['Value']) {
+	while (cacti_sizeof($delayed_inserts) && $delayed_inserts['Value']) {
 		cacti_log('BOOST WAIT: Waiting 1s for delayed inserts are made' , true, 'SYSTEM');
 		usleep(1000000);
 		$delayed_inserts = db_fetch_row("SHOW STATUS LIKE 'Not_flushed_delayed_rows'");
@@ -333,11 +335,12 @@ function boost_prepare_process_table() {
 
 	db_execute("CREATE TABLE $interim_table LIKE poller_output_boost");
 	db_execute("RENAME TABLE poller_output_boost TO $archive_table, $interim_table TO poller_output_boost");
+	db_execute("ANALYZE TABLE $archive_table");
 
-	$arch_tables = boost_get_arch_table_names();
+	$arch_tables = boost_get_arch_table_names($archive_table);
 
 	if (!cacti_sizeof($arch_tables)) {
-		cacti_log('ERROR: Failed to retrieve archive table name', false, 'BOOST');
+		cacti_log('ERROR: Failed to retrieve archive table name - check poller', false, 'BOOST');
 
 		return false;
 	}
@@ -365,7 +368,7 @@ function boost_prepare_process_table() {
 		process_handler int unsigned default "0",
 		PRIMARY KEY (local_data_id),
 		INDEX process_handler(process_handler))
-		ENGINE=MEMORY');
+		ENGINE=InnoDB');
 
 	db_execute('TRUNCATE poller_output_boost_local_data_ids');
 
@@ -439,20 +442,6 @@ function boost_launch_children() {
 	sleep(2);
 }
 
-function boost_debug($string) {
-	global $debug, $child;
-
-	$string = 'DEBUG: ' . trim($string, " \n");
-
-	if ($debug) {
-		print $string . PHP_EOL;
-
-		if ($child) {
-			cacti_log($string, false, 'BOOST CHILD');
-		}
-	}
-}
-
 function boost_time_to_run($forcerun, $current_time, $last_run_time, $next_run_time) {
 	$run_now = false;
 
@@ -460,8 +449,8 @@ function boost_time_to_run($forcerun, $current_time, $last_run_time, $next_run_t
 
 	if ((read_config_option('boost_rrd_update_enable') == 'on') || $forcerun) {
 		/* turn on the system level updates as that is what dictates "on/off" */
-		if ((!$forcerun) && (read_config_option('boost_rrd_update_system_enable') != 'on')) {
-			set_config_option('boost_rrd_update_system_enable','on');
+		if (!$forcerun && read_config_option('boost_rrd_update_system_enable') != 'on') {
+			set_config_option('boost_rrd_update_system_enable', 'on');
 		}
 
 		$seconds_offset = read_config_option('boost_rrd_update_interval') * 60;
@@ -531,7 +520,7 @@ function boost_time_to_run($forcerun, $current_time, $last_run_time, $next_run_t
 }
 
 function boost_output_rrd_data($child) {
-	global $start, $max_run_duration, $config, $database_default, $debug, $get_memory, $memory_used;
+	global $start, $archive_table, $max_run_duration, $config, $database_default, $debug, $get_memory, $memory_used;
 
 	$rrd_updates       = 0;
 	$rrdtool_pipe      = rrd_init();
@@ -547,7 +536,7 @@ function boost_output_rrd_data($child) {
 
 	boost_debug("Processing RRRtool Output for Boost Process $child");
 
-	$arch_tables = boost_get_arch_table_names();
+	$arch_tables = boost_get_arch_table_names($archive_table);
 
 	if (!cacti_sizeof($arch_tables)) {
 		cacti_log('ERROR: Failed to retrieve archive table name', false, 'BOOST');
@@ -636,10 +625,10 @@ function boost_output_rrd_data($child) {
    @arg $child - the current process
    @arg $rrdtool_pipe - the socket that has been opened for the RRDtool operation */
 function boost_process_local_data_ids($last_id, $child, $rrdtool_pipe) {
-	global $config, $boost_sock, $boost_timeout, $debug, $get_memory, $memory_used, $current_lock;
+	global $config, $archive_table, $boost_sock, $boost_timeout, $debug, $get_memory, $memory_used, $current_lock;
 
 	/* cache this call as it takes time */
-	static $archive_tables   = false;
+	static $archive_tables  = false;
 	static $rrdtool_version = '';
 
 	include_once($config['library_path'] . '/rrd.php');
@@ -674,11 +663,11 @@ function boost_process_local_data_ids($last_id, $child, $rrdtool_pipe) {
 	$data_ids_to_get     = read_config_option('boost_rrd_update_max_records_per_select');
 
 	if ($archive_tables === false) {
-		$archive_tables = boost_get_arch_table_names();
+		$archive_tables = boost_get_arch_table_names($archive_table);
 	}
 
 	if ($archive_tables === false) {
-		boost_debug('Failed to determine archhive tables');
+		boost_debug('Failed to determine archive tables');
 		cacti_log('Failed to determine archive tables', false, 'BOOST');
 		return 0;
 	}
@@ -733,7 +722,7 @@ function boost_process_local_data_ids($last_id, $child, $rrdtool_pipe) {
 			$item['timestamp'] = trim($item['timestamp']);
 
 			if (!$locked) {
-				/* aquire lock in order to prevent race conditions, only a problem pre-rrdtool 1.5 */
+				/* acquire lock in order to prevent race conditions, only a problem pre-rrdtool 1.5 */
 				while (!db_fetch_cell("SELECT GET_LOCK('boost.single_ds." . $item['local_data_id'] . "', 1)")) {
 					usleep(50000);
 				}
@@ -752,7 +741,7 @@ function boost_process_local_data_ids($last_id, $child, $rrdtool_pipe) {
 
 				$current_lock = false;
 
-				/* aquire lock in order to prevent race conditions, only a problem pre-rrdtool 1.5 */
+				/* acquire lock in order to prevent race conditions, only a problem pre-rrdtool 1.5 */
 				while (!db_fetch_cell("SELECT GET_LOCK('boost.single_ds." . $item['local_data_id'] . "', 1)")) {
 					usleep(50000);
 				}
@@ -929,18 +918,18 @@ function boost_process_local_data_ids($last_id, $child, $rrdtool_pipe) {
 		$current_lock = false;
 
 		boost_timer('results_cycle', BOOST_TIMER_END);
-
-		/* remove the entries from the table */
-		boost_timer('delete', BOOST_TIMER_START);
-
-		db_execute_prepared("DELETE FROM poller_output_boost_local_data_ids
-			WHERE local_data_id <= ?
-			AND process_handler = ?",
-			array($last_id, $child));
-
-		boost_timer('delete', BOOST_TIMER_END);
 	}
 
+	/* remove the entries from the table */
+	boost_timer('delete', BOOST_TIMER_START);
+
+	db_execute_prepared("DELETE FROM poller_output_boost_local_data_ids
+		WHERE local_data_id <= ?
+		AND process_handler = ?",
+		array($last_id, $child));
+
+	boost_timer('delete', BOOST_TIMER_END);
+
 	/* restore original error handler */
 	restore_error_handler();
 
