@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -51,6 +51,7 @@ $debug    = false;
 $force    = false;
 $archived = 0;
 $purged   = 0;
+$start    = microtime(true);
 
 if (cacti_sizeof($parms)) {
 	foreach ($parms as $parameter) {
@@ -99,6 +100,8 @@ if ($config['poller_id'] == 1) {
 	authcache_purge();
 
 	secpass_check_expired();
+
+	reindex_devices();
 }
 
 // Check the realtime cache and poller
@@ -110,10 +113,81 @@ api_device_purge_deleted_devices();
 // Rotate Cacti Logs
 logrotate_check($force);
 
+// Remove deleted devices
+remove_aged_row_cache();
+
+if ($config['poller_id'] > 1) {
+	api_plugin_hook('poller_remote_maint');
+}
+
+$end = microtime(true);
+
+cacti_log(sprintf('MAINT STATS: Time:%0.2f', $end - $start), false, 'SYSTEM');
+
 unregister_process('maintenance', 'master', $config['poller_id']);
 
 exit(0);
 
+function reindex_devices() {
+	global $config;
+
+	$schedule = read_config_option('automatic_reindex');
+
+	// 0 - Disabled
+	// 1 - Daily at Midnight
+	// 2 - Weekly on Sunday
+	// 3 - Monthly on Sunday
+
+	$command_string = cacti_escapeshellcmd(read_config_option('path_php_binary'));
+	$extra_args     = $config['base_path'] . '/cli/poller_reindex_hosts.php --id=all --qid=all';
+
+	if ($schedule == 0) {
+		return;
+	}
+
+	$last_run = read_config_option('periodic_reindex_lastrun');
+	$now      = time();
+
+	if (empty($last_run)) {
+		set_config_option('periodic_reindex_lastrun', time());
+		return;
+	} else {
+		if ($schedule == 1) {
+			if (date('z', $now) != date('z', $last_run)) {
+				set_config_option('periodic_reindex_lastrun', $now);
+				exec_background($command_string, $extra_args);
+			}
+		} elseif ($schedule == 2) {
+			if (date('z', $now) != date('z', $last_run) && date('w', $now) == 0) {
+				exec_background($command_string, $extra_args);
+			}
+		} elseif ($schedule == 3) {
+			if (date('z', $now) != date('z', $last_run)) {
+				if (date('w', $now) == 0 && date('n', $now) != date('n', $last_run)) {
+					exec_background($command_string, $extra_args);
+				}
+			}
+		}
+	}
+}
+
+function remove_aged_row_cache() {
+	$classes = array_rekey(
+		db_fetch_assoc('SELECT REPLACE(name, "time_last_change_", "") AS name, value
+			FROM settings
+			WHERE name LIKE "time_last_change%"'),
+		'name', 'value'
+	);
+
+	if (cacti_sizeof($classes)) {
+		foreach($classes as $name => $ts) {
+			db_execute_prepared('DELETE FROM user_auth_row_cache
+				WHERE class = ? AND UNIX_TIMESTAMP(time) < ?',
+				array($name, $ts));
+		}
+	}
+}
+
 function logrotate_check($force) {
 	global $disable_log_rotation;
 
@@ -584,7 +658,7 @@ function rrdclean_create_path($path) {
 				$group_id      = filegroup($config['rra_path']);
 
 				// NOTE: chown/chgrp fails for non-root users, checking their
-				// result is therefore irrevelevant
+				// result is therefore irrelevant
 				@chown($path, $owner_id);
 				@chgrp($path, $group_id);
 			}
@@ -682,7 +756,7 @@ function display_help() {
 	display_version();
 
 	print "\nusage: poller_maintenance.php [--force] [--debug]\n\n";
-	print "Cacti's maintenance poller.  This poller is repsonsible for executing periodic\n";
+	print "Cacti's maintenance poller.  This poller is responsible for executing periodic\n";
 	print "maintenance activities for Cacti including log rotation, deactivating accounts, etc.\n\n";
 	print "Optional:\n";
 	print "    --force   - Force immediate execution, e.g. for testing.\n";
