@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 #  +-------------------------------------------------------------------------+
-#  | Copyright (C) 2004-2022 The Cacti Group                                 |
+#  | Copyright (C) 2004-2023 The Cacti Group                                 |
 #  |                                                                         |
 #  | This program is free software; you can redistribute it and/or           |
 #  | modify it under the terms of the GNU General Public License             |
@@ -224,7 +224,7 @@ save_log_files() {
 			cp -f "$logFile1" "${logBase}/wget_error.log"
 		fi
 
-		chmod a+r "${logBase}/*.log"
+		chmod a+r -R "${logBase}/"
 
 		if [ $DEBUG -eq 1 ];then
 			echo "DEBUG: Dumping ${CACTI_LOG}"
